      module inparms
        type gficard  ! Define a new type for the input parms
          sequence
          real      undef  ! Undefined value for the grid
          integer   itype  ! i dimension:  0 = X-dimension
          integer   jtype  ! j dimension:  0 = Y-dimension
          integer   nx     ! num pts in i-direction
          integer   ny     ! num pts in j-direction
          integer   numpts ! num pts on grid (nx*ny)
          real      xbegi  ! lon of first point
          real      dxi    ! grid increment in i-direction
          real      ybegi  ! lat of first point
          real      dyi    ! grid increment in j-direction
        end type gficard
      end module inparms

      module tracked_parms
          real, save, allocatable  ::  xdata_in(:)
          real, save, allocatable  ::  xin(:)
          real, save, allocatable  ::  yin(:)
          real, save, allocatable  ::  xdata_2d(:,:)
          real, save, allocatable  ::  xlon(:)
          real, save, allocatable  ::  ylat(:)
      end module tracked_parms

      module trig_vals
        real, save :: pi, dtr
        real, save :: dtk = 111.1949     ! Dist (km) over 1 deg lat
                                         ! using erad=6371.0e+3
        real, save :: erad = 6371.0e+3   ! Earth's radius (m)
        real, save :: ecircum = 40030.2  ! Earth's circumference
                                         ! (km) using erad=6371.e3
        real, save :: omega = 7.292e-5
      end module trig_vals

c
c---------------------------------------------------------------------c
c
      program findctr
c
c     ABSTRACT: This GrADS UDF scans an array of data values to find
c     the max or min in the vicinity of a user-provided guess position.
c     This program assumes that the user has clicked a position that is
c     close to some type of storm, whether that be represented as a 
c     min in the mslp field, max in the 850 zeta field, etc.  This 
c     program will center its search on that user-provided position.
c
c     Author:    Tim Marchok  (timothy.marchok@noaa.gov)
c     Date:      August, 2005
c     Language:  Fortran90
c
c---------------------------------------------------------------------c
c
      USE inparms; USE tracked_parms; USE trig_vals
      implicit none

      type (gficard) inp

      real vals(20),ovals(20)
      real guesslon,guesslat,ctlon,ctlat,xval
      integer, parameter :: iunit_gr_in=21,iunit_gr_out=51
      integer, parameter :: iunit_diag=52,iunit_ctr=53
      integer    nargs,irvret,iopret,icrret,ifmret
      character  cmaxmin*3,cparm*4,cres*5

      data ovals/20*0.0/
c
      pi = 4. * atan(1.)   ! Both pi and dtr were declared in module
      dtr = pi/180.0       ! trig_vals, but were not yet defined.

      call open_files (iunit_gr_in,iunit_gr_out,iunit_diag,iunit_ctr
     &                ,ovals)

      call read_vals (vals,ovals,iunit_gr_in,iunit_diag,inp,irvret
     &               ,guesslon,guesslat,cmaxmin,cparm,cres)

      call conv1d2d_real (inp,icrret)

      call find_maxmin (inp%nx,inp%ny,inp%dxi,cparm,guesslon,guesslat
     &                 ,ctlon,ctlat,xval,iunit_diag,inp,cres,ifmret)

      call write_vals (vals,ovals,iunit_gr_out,iunit_ctr,ctlon,ctlat
     &                 ,iunit_diag,cparm)

c
      if (allocated(xdata_in)) deallocate (xdata_in)
      if (allocated(xin))      deallocate (xin)
      if (allocated(yin))      deallocate (yin)

      close (iunit_gr_in)
      close (iunit_gr_out)
      close (iunit_diag)
      close (iunit_ctr)
c
      stop 
      end

c----------------------------------------------------------------
c
c----------------------------------------------------------------
      subroutine open_files (iunit_gr_in,iunit_gr_out,iunit_diag
     &                      ,iunit_ctr,ovals)
c
c     ABSTRACT: This subroutine opens up the various input and 
c     output files needed for this GrADS UDF.
c
      implicit none

      real     ovals(20)
      integer  iunit_gr_in,iunit_gr_out,iunit_diag,iunit_ctr
c
      open (iunit_gr_in,file='udf.findctr.gfi',status='unknown',
     &     form='unformatted',err=910)

      open (iunit_gr_out,file='udf.findctr.gfo',status='unknown',
     &     form='unformatted',err=920)

      open (iunit_diag,file='udf.findctr.out',
     &     form='formatted',status='unknown',err=930)

      open (iunit_ctr,file='udf.fixfile.txt',
     &     form='formatted',status='unknown',err=940)

      goto 400
c
  910 ovals(1) = 910.
      write(iunit_diag,*) '!!! ERROR: findctr error 910, unable to open'
     &                   ,' grads input file to findctr'
      stop 95
  920 ovals(1) = 920.
      write(iunit_diag,*) '!!! ERROR: findctr error 920, unable to open'
     &                   ,' grads output gfo file for findctr'
      stop 95
  930 ovals(1) = 930.
      write(iunit_diag,*) '!!! ERROR: findctr error 930, unable to open'
     &                   ,' grads standard output file for findctr'
      stop 95
  940 ovals(1) = 940.
      write(iunit_diag,*) '!!! ERROR: findctr error 940, unable to open'
     &                   ,' grads center fix text file for findctr'
      stop 95
c
  400 continue
c
      return
      end

c-------------------------------------------------------------------
c
c-------------------------------------------------------------------
      subroutine read_vals (vals,ovals,iunit_gr_in,iunit_diag,inp
     &               ,irvret,guesslon,guesslat,cmaxmin,cparm,cres)
c
c     ABSTRACT: This subroutine reads the input vals.  There is 
c     an order to the reads that is partly determined by GrADS
c     "user-defined function" conventions and partly by the 
c     ordering of the calling arguments to findctr.  The first
c     record read is always a standard header record, then each
c     record that is read after that is related to the number of
c     and the ordering of the input arguments (see the GrADS 
c     documentation on User Defined Functions for more details,
c     specifically the section titled, "Format of the function 
c     data transfer file".

      USE inparms; USE tracked_parms
      implicit none

      type (gficard) inp

      real      vals(20),ovals(20)
      real      guesslon,guesslat
      integer   nargs,iunit_gr_in,iunit_diag,irvret,ixdi,ixi,iyi,i
      character cmaxmin*3,cparm*4,cres*5
c

c     ----------------------------------------------------
c     The first record is the header record and contains 
c     the number of args and other parameters that may be
c     used for future implementations.

      read (iunit_gr_in) vals
      nargs=nint(vals(1))
      write(iunit_diag,*) 'number of arguments = ',nargs

c     Stop if number of input arguments not equal to 6....

      if (nargs /= 6) then
        ovals(1)=1.0
        write(*,12)
 12     format(' ',/
     $       /' ','findctr requires exactly 6 arguments!!',
     $       /' ',' ')
        stop 94
      endif

c     ----------------------------------------------------
c     The second set of reads is for the data field ("expr" 
c     in the user defined function table) and is input using 
c     four read statements, as follows....

c     Grid record 1: Essentially another header record, with 
c     specific information about the global attributes of the
c     gridded data.

      read (iunit_gr_in,err=820) vals

c     Parse out the grid parameters from the vals just read in....

      inp%undef  = vals(1)
      inp%itype  = nint(vals(2))
      inp%jtype  = nint(vals(3))
      inp%nx     = nint(vals(4))
      inp%ny     = nint(vals(5))
      inp%numpts = inp%nx*inp%ny
      inp%xbegi  = vals(8)
      inp%dxi    = vals(9)
      inp%ybegi  = vals(10)
      inp%dyi    = vals(11)

      write (iunit_diag,*) 'undef(1)=       ',inp%undef
      write (iunit_diag,*) 'itype(2)=       ',inp%itype
      write (iunit_diag,*) 'jtype(3)=       ',inp%jtype
      write (iunit_diag,*) 'nx(4)=          ',inp%nx
      write (iunit_diag,*) 'ny(5)=          ',inp%ny
      write (iunit_diag,*) 'numpts=         ',inp%numpts
      write (iunit_diag,*) 'iscalar flag =  ',vals(6)
      write (iunit_diag,*) 'jscalar flag =  ',vals(7)
      write (iunit_diag,*) 'xbegi(8)=       ',inp%xbegi
      write (iunit_diag,*) 'dxi(9)=         ',inp%dxi
      write (iunit_diag,*) 'ybegi(10)=      ',inp%ybegi
      write (iunit_diag,*) 'dyi(11)=        ',inp%dyi

c     Grid record 2: The data values

      allocate (xdata_in(inp%numpts),stat=ixdi)
      allocate (xin(inp%nx),stat=ixi)
      allocate (yin(inp%ny),stat=iyi)

      if (ixdi /= 0 .or. ixi /= 0 .or. iyi /= 0) then
        print *,' '
        print *,'!!! ERROR in sub read_vals allocating arrays.'
        print *,'!!! ixdi = ',ixdi,' ixi= ',ixi,' iyi= ',iyi
        irvret = 94
        return
      endif

      read (iunit_gr_in,err=830) xdata_in
      do i = 1,20
        write (iunit_diag,105) i,'  xdata_in= ',xdata_in(i)
      enddo
      read (iunit_gr_in,err=840) xin
      do i = 1,20
        write (iunit_diag,105) i,'       xin= ',xin(i)
      enddo
      read (iunit_gr_in,err=850) yin
      do i = 1,20
        write (iunit_diag,105) i,'       yin= ',yin(i)
      enddo

  105 format (1x,'i= ',i3,a12,f10.3)

c     Now read the remaining records.  There will be 1 record 
c     for each additional argument that is passed to findctr.

      read (iunit_gr_in,err=860) guesslon
      read (iunit_gr_in,err=860) guesslat
      read (iunit_gr_in,err=860) cmaxmin
      read (iunit_gr_in,err=860) cparm
      read (iunit_gr_in,err=860) cres
      
      write (iunit_diag,111) guesslon,guesslat
      write (iunit_diag,113) cmaxmin,cparm
      write (iunit_diag,115) cres

  111 format (1x,'+++ INPUT: guesslon= ',f8.3,'  guesslat= ',f8.3)
  113 format (1x,'+++ INPUT: Max/Min flag= ',a3
     &               ,'  Parm being searched= ',a4)
  115 format (1x,'+++ INPUT: grid resolution= ',a5)

      goto 400
c
  820 ovals(1) = 820.
      write(iunit_diag,*) '!!! ERROR: findctr error 820.... Error while'
     &                   ,' reading grads input file to findctr'
      stop 95
  830 ovals(1) = 830.
      write(iunit_diag,*) '!!! ERROR: findctr error 830.... Error while'
     &                   ,' reading grads input file to findctr'
      stop 95
  840 ovals(1) = 840.
      write(iunit_diag,*) '!!! ERROR: findctr error 840.... Error while'
     &                   ,' reading grads input file to findctr'
      stop 95
  850 ovals(1) = 850.
      write(iunit_diag,*) '!!! ERROR: findctr error 850.... Error while'
     &                   ,' reading grads input file to findctr'
      stop 95
  860 ovals(1) = 860.
      write(iunit_diag,*) '!!! ERROR: findctr error 860.... Error while'
     &                   ,' reading grads input file to findctr'
      stop 95
c
  400 continue
      return
      end

c-----------------------------------------------------------------------
c
c-----------------------------------------------------------------------
      subroutine write_vals (vals,ovals,iunit_gr_out,iunit_ctr
     &                      ,ctlon,ctlat,iunit_diag,cparm)
c
c     ABSTRACT: This subroutine writes out the data needed for the GrADS
c     function result file.  We don't really need those values, but 
c     GrADS requires that they be written out.  We do, however, need the
c     value of the center lat & lon that were found.

      USE tracked_parms
      implicit none

      real     vals(20),ovals(20)
      real     ctlon,ctlat
      integer  iunit_gr_out,iunit_ctr,iunit_diag
      character cparm*4

c     Write the values back out to a GrADS function result file....

      write (iunit_gr_out) ovals
      write (iunit_gr_out) vals
      write (iunit_gr_out) xdata_2d

c     Write the values of the found center lat/lon positions out
c     to a text data file....

      write (iunit_ctr,101) ctlon,ctlat
  101 format(1x,'center_lat= ',f8.3,'  center_lon= ',f8.3)

      write (iunit_diag,103) cparm,ctlon,ctlat
      write (*,103) cparm,ctlon,ctlat
  103 format(1x,'tracker fix for parm -->',a4,'<--  center_lat= ',f8.3
     &         ,'  center_lon= ',f8.3)
c  
      return
      end

c-----------------------------------------------------------------------
c
c-----------------------------------------------------------------------
      subroutine conv1d2d_real (inp,icrret)
c
c     ABSTRACT: This subroutine converts a 1-dimensional input
c     array of real data into a 2-dimensional output array of 
c     real data.  For use with GFDL data, point (1,1) is in 
c     the far SW corner and point (imax,jmax) is in the far
c     NE corner of the grid, but because we are using center-fixing
c     code that I pulled out of my tracker which requires that point
c     (1,1) be in the far NW corner and point (imax,jmax) in the 
c     far SE corner (i.e., flipped), we will flip the arrays in this
c     subroutine.

      USE inparms; USE tracked_parms
      implicit none
      
      type (gficard) inp

      integer   ixdi,ixi,iyi,icrret
      integer   jlatix,jlat,ilon
c
      allocate (xdata_2d(inp%nx,inp%ny),stat=ixdi)
      allocate (xlon(inp%nx),stat=ixi)
      allocate (ylat(inp%ny),stat=iyi)

      if (ixdi /= 0 .or. ixi /= 0 .or. iyi /= 0) then
        print *,' '
        print *,'!!! ERROR in sub conv1d2d_real allocating arrays.'
        print *,'!!! ixdi = ',ixdi,' ixi= ',ixi,' iyi= ',iyi
        icrret = 94
        return
      endif

c     Input data is south to north; flip the data while
c     converting to 2-d grid.  At the same time, fill the ylat 
c     array with values so that ylat(1) = the lat value of the
c     northernmost point.

      do jlat=1,inp%ny
        jlatix = inp%ny - jlat + 1
        ylat(jlat) = yin(jlatix)
        do ilon=1,inp%nx
          xlon(ilon) = xin(ilon)
          xdata_2d(ilon,jlatix) = xdata_in(ilon+(jlat-1)*inp%nx)
        enddo
      enddo
c 
      return
      end

c-----------------------------------------------------------------------
c
c-----------------------------------------------------------------------
      subroutine find_maxmin (imax,jmax,grdspc,cparm,guesslon,guesslat
     &                    ,ctlon,ctlat,xval,iunit_diag,inp,cres,ifmret)
c
c     This routine  finds the location (clon,clat) of and value of the
c     the max or min of fxy in the vicinity of slon,slat.  The value of
c     the input flag maxmin determines whether to look for a max or a
c     min value.  The max/min is determined by finding the point which 
c     gives the max/min value of a single point barnes analysis of fxy  
c     with e-folding radius re (km) and influence radius ri (km). The 
c     initial search is restricted to a radius rads around the point 
c     (slon,slat) on a grid with lon,lat spacing grdspc. The location is
c     refined by reducing the spacing of the search grid by a factor of
c     two, nhalf times.
c
c     INPUT:  
c     imax     Num pts in i direction on input grid
c     jmax     Num pts in j direction on input grid
c     grdspc   Grid spacing on input grid
c     cparm    Char string indicating what parm is being passed in
c     guesslon Guess longitude of the storm
c     guesslat Guess latitude of the storm
c
c     OUTPUT: 
c     ctlon    Center longitude of storm found for this parameter
c     ctlat    Center latitude of storm found for this parameter
c     xval     Max or Min value found at the (ctlon,ctlat)
c     ifmret   Return code from this subroutine

      USE inparms; USE tracked_parms; USE trig_vals
      implicit none

      type (gficard) inp

c      real, parameter :: retrk_lo=75.0
c      real, parameter :: ritrk_lo=150.0
c      real, parameter :: rads_lo=300.0
c      real, parameter :: retrk_hi=75.0
c      real, parameter :: ritrk_hi=150.0
c      real, parameter :: rads_hi=150.0

      real, parameter :: retrk_lo=50.0
      real, parameter :: ritrk_lo=125.0
      real, parameter :: rads_lo=300.0
      real, parameter :: retrk_hi=50.0
      real, parameter :: ritrk_hi=125.0
      real, parameter :: rads_hi=150.0

      integer, parameter :: nhalf=3
      integer   jlatix,jlat,ilon,imax,jmax,ifmret,iunit_diag,npts
      integer   ifix,jfix,ibct,ibarnes_loopct,jix,iix,i,j,icount
      integer   ibeg,iend,jbeg,jend,iret,k
      real      guesslon,guesslat,ctlon,ctlat,xval,grdspc,dell,ftemp
      real      fmax,fmin,rlont,rlatt,dist,tlon,tlat,rads,re,ri
      real      retmp,ritmp
      real      fxy(imax,jmax)
      character cparm*4,cmaxmin*3,cres*5
c
      if (cres == 'hires') then
        rads  = rads_hi 
        re    = retrk_hi
        ri    = ritrk_hi
      else
        rads  = rads_lo 
        re    = retrk_lo
        ri    = ritrk_lo
      endif

      dell  = grdspc
      fmax  = -1.0e+15; fmin  =  1.0e+15
      ctlon = 0.0; ctlat = 0.0

      if (cparm == 'psml') cmaxmin = 'min'
      if (cparm == 'zeta') cmaxmin = 'max'

c     Fix the guess lat/lon point to the nearest (i,j) point.

      call fix_latlon_to_ij (imax,jmax,grdspc,guesslon,guesslat
     &          ,ifix,jfix,inp,iunit_diag,'A')

c     Figure out the number of data points to search
      
      npts = int(rads/(dtk*dell))
      if (npts == 0) npts = 1

      write (iunit_diag,201) imax,jmax,grdspc
      write (iunit_diag,203) guesslon,guesslat,ifix,jfix
      write (iunit_diag,301) npts

  201 format (1x,'A imax= ',i4,' jmax= ',i4,' grdspc= ',f8.4)
  203 format (1x,'A guesslon= ',f8.3,' guesslat= ',f8.3,' ifix= '
     &       ,i4,' jfix= ',i4)
  301 format (1x,'Before first loop, npts= ',i5)

c     Now loop through, calling the  barnes routine for all npts
c     surrounding our guess position....

      ibct = 0
      ibarnes_loopct = 0
      
      jix = 0
      
      jloop: do j=-npts,npts
        
        jix = jix + 1
        rlatt = guesslat + dell*float(j)

        if (rlatt > ylat(1) .or. rlatt < ylat(inp%ny)) then
          write (iunit_diag,*) '!!! find_maxmin cycle on rlatt.  '
     &           ,' rlatt= ',rlatt,' ylat(1)= ',ylat(1)
     &           ,' ylat(max)= ',ylat(inp%ny)
          cycle jloop
        endif
        
        iix = 0

        iloop: do i=-npts,npts

          iix = iix + 1
          rlont = guesslon + dell*float(i)

          if (rlont < xlon(1) .or. rlont > xlon(inp%nx)) then
            write (iunit_diag,*) '!!! find_maxmin cycle on rlont.  '
     &             ,' rlont= ',rlont,' xlon(1)= ',xlon(1)
     &             ,' xlon(max)= ',xlon(inp%ny)
            cycle iloop
          endif

          call calcdist(rlont,rlatt,guesslon,guesslat,dist)
          if (dist > rads) cycle iloop

c         Fix the rlont/rlatt point to the nearest (i,j) point, then
c         determine the beginning and ending points for the  barnes 
c         analysis.

          call fix_latlon_to_ij (imax,jmax,grdspc,rlont,rlatt
     &                          ,ifix,jfix,inp,iunit_diag,'B')

          ibeg = ifix - npts
          iend = ifix + npts
          jbeg = jfix - npts
          jend = jfix + npts

          if (ibeg < 1)      ibeg = 1
          if (iend > inp%nx) iend = inp%nx
          if (jbeg < 1)      jbeg = 1
          if (jend > inp%ny) jend = inp%ny

c          write (iunit_diag,204) 
c          write (iunit_diag,205) i,j,rlont,rlatt
c          write (iunit_diag,207) i,j,ibeg,iend,jbeg,jend

  204     format (' ')
  205     format (1x,'B i= ',i4,' j= ',i4,' rlont= ',f8.3,' rlatt= '
     &           ,f8.3)
  207     format (1x,'B i= ',i4,' j= ',i4,' ibeg= ',i4,' iend= ',i4
     &           ,' jbeg= ',i4,' jend= ',i4)

          ibct = ibct + 1
          call barnes(rlont,rlatt,imax,jmax,ibeg,jbeg,iend,jend
     &               ,re,ri,ftemp,icount,cparm,iret)

          ibarnes_loopct = ibarnes_loopct + icount

          if (iret /= 0) then
            print *,' '
            print *,'!!! Non-zero RCC from barnes...Exiting find_maxmin'
            ifmret = iret
            return
          endif
c
          if (cmaxmin == 'max') then
            if (ftemp > fmax) then
              fmax = ftemp
              ctlon = rlont
              ctlat = rlatt
            endif
          else
            if (ftemp < fmin) then
              fmin = ftemp
              ctlon = rlont
              ctlat = rlatt
            endif
          endif

        enddo iloop
      enddo jloop

      write (iunit_diag,*) ' '
      write (iunit_diag,*) 'After 1st findmax loop, # calls to barnes'
     &                     ,' = ',ibct
      write (iunit_diag,*) 'Total # of barnes loop iterations = '
     &                     ,ibarnes_loopct

      if (cparm == 'zeta') then
        write (iunit_diag,*) ' !!! Zeta check, fmax= ',fmax,' fmin= '
     &                      ,fmin
        write (iunit_diag,61) ctlon*-1.,ctlat,fmax*100000.,fmin*100000.
        write (6,61) ctlon*-1.,ctlat,fmax*100000.,fmin*100000.
      else
        write (iunit_diag,63) ctlon*-1.,ctlat,fmin
        write (6,63) ctlon*-1.,ctlat,fmin
      endif
  61  format (' After first run, ctlon= ',f8.3,'W  ctlat= ',f8.3
     &       ,' fmax (x10e5) = ',f10.3,' fmin (x10e5) = ',f10.3)
  63  format (' After first run, ctlon= ',f8.3,'W  ctlat= ',f8.3
     &       ,' fmin = ',f10.3)
 111  format (i2,'  rlont= ',f7.2,'W   rlatt= ',f7.2,'  zeta= ',f13.8)

c     --------------------------------------------------------------
c     Halve the grid spacing to refine the location and value of the
c     max/min value (dell = 0.5*dell), but restrict the area of the 
c     new search grid (npts = npts/2).

      npts = npts / 2

      call fix_latlon_to_ij (imax,jmax,grdspc,ctlon,ctlat
     &                      ,ifix,jfix,inp,iunit_diag,'C')

c     ------------------------------------
c     Now do the actual searching for the 
c     max/min value

      write (iunit_diag,*) ' '
      if (grdspc <= 1.25 .and. ri >= 300 .and. re >= 150) then
        retmp = re
        ritmp = ri
        re = re * 0.5
        ri = ri * 0.5
        write (iunit_diag,*) 'After first pass through barnes, re has '
     &                      ,'been reduced'
        write (iunit_diag,*) 'from ',retmp,' to ',re,', and ri has been'
     &                      ,' reduced '
        write (iunit_diag,*) 'from ',ritmp,' to ',ri
      else
        write (iunit_diag,*) 'After first pass through barnes, re and '
     &                      ,'ri have NOT '
        write (iunit_diag,*) 'been changed.  re = ',re,' ri = ',ri
      endif

      write (iunit_diag,303) npts
  303 format (1x,'Before nhalf loop, npts= ',i5)

      ibct=0
      ibarnes_loopct = 0
      do k=1,nhalf

        dell = 0.5*dell
        tlon = ctlon
        tlat = ctlat
        fmax = -1.0e+15; fmin = 1.0e+15

        write (iunit_diag,*) ' '
        write (iunit_diag,*) 'find_maxmin nhalf loop, cparm= ',cparm
     &                      ,' k= ',k
        write (iunit_diag,161) tlon,tlon*-1.,tlat
        write (iunit_diag,*) 'ifix= ',ifix,' jfix= ',jfix,' npts= ',npts
        write (iunit_diag,*) 'ibeg= ',ibeg,' jbeg= ',jbeg,' imax= ',imax
        write (iunit_diag,*) 'iend= ',iend,' jend= ',jend,' jmax= ',jmax

        jloop2: do j = -npts,npts

          rlatt = tlat + dell*float(j)

          if (rlatt > ylat(1) .or. rlatt < ylat(inp%ny)) then
            write (iunit_diag,*) '!!! find_maxmin cycle on rlatt. '
     &             ,' rlatt= ',rlatt,' ylat(1)= ',ylat(1),' ylat(max)= '
     &             ,ylat(inp%ny)
            cycle jloop2
          endif

          iloop2: do i = -npts,npts

            rlont = tlon + dell*float(i)

            if (rlont < xlon(1) .or. rlont > xlon(inp%nx)) then
              write (iunit_diag,*) '!!! find_maxmin cycle on rlont. '
     &               ,' rlont= ',rlont,' xlon(1)= ',xlon(1)
     &               ,' xlon(max)= ',xlon(inp%ny)
              cycle iloop2
            endif

c           Fix the rlont/rlatt point to the nearest (i,j) point, then
c           determine the beginning and ending points for the  barnes 
c           analysis.

            call fix_latlon_to_ij (imax,jmax,grdspc,rlont,rlatt 
     &                            ,ifix,jfix,inp,iunit_diag,'D')

            ibeg = ifix - npts
            iend = ifix + npts
            jbeg = jfix - npts
            jend = jfix + npts
     
            if (ibeg < 1)      ibeg = 1
            if (iend > inp%nx) iend = inp%nx
            if (jbeg < 1)      jbeg = 1 
            if (jend > inp%ny) jend = inp%ny

            ibct = ibct + 1
            call barnes(rlont,rlatt,imax,jmax,ibeg,jbeg,iend,jend
     &                 ,re,ri,ftemp,icount,cparm,iret)

            ibarnes_loopct = ibarnes_loopct + icount

            if (iret /= 0) then
              print *,' '
              print *,'!!! Non-zero RCC from barnes, k= ',k
              print *,'!!! Exiting find_maxmin'
              ifmret = iret
              return
            endif

            if (cmaxmin == 'max') then
              if (ftemp > fmax) then
                fmax = ftemp
                ctlon = rlont
                ctlat = rlatt
              endif
            else
              if (ftemp < fmin) then
                fmin = ftemp
                ctlon = rlont
                ctlat = rlatt
              endif
            endif

          enddo iloop2
        enddo jloop2
        
        if (cparm == 'zeta') then
          write (6,71) k,ctlon*-1.,ctlat,fmax*100000.,fmin*100000.
        else
          write (6,73) k,ctlon*-1.,ctlat,fmin 
        endif
        
      enddo

  71  format (' nhalf findmax, k= ',i2,' ctlon= ',f8.3,'W  ctlat= '
     &       ,f8.3,' fmax (x10e5) = ',f10.3,' fmin (x10e5) = ',f10.3)
  73  format (' nhalf findmax, k= ',i2,' ctlon= ',f8.3,'W  ctlat= '
     &       ,f8.3,' fmin = ',f10.3)

 161  format (' guesslon= ',f8.3,'E  (',f8.3,'W)   guesslat= ',f8.3)
c
      return
      end

c----------------------------------------------------------------------
c
c----------------------------------------------------------------------
      subroutine barnes(flon,flat,iimax,jjmax,iibeg,jjbeg,iiend,jjend
     &                 ,re,ri,favg,icount,cparm,iret)
c
c     ABSTRACT: This routine performs a single-pass barnes anaylsis
c     of a data field at the point (flon,flat). The e-folding radius 
c     (km) and influence radius (km) are re and ri, respectively.
c
c     NOTE:  The input grid that is searched in this subroutine is most
c     likely NOT the model's full, original grid.  Instead, a smaller
c     subgrid of the original grid is searched.  The upper left and
c     lower right grid point indices are passed into this subroutine
c     (iibeg, jjbeg, iiend, jjend) for this subgrid.  These indices are
c     determined in the subroutine  get_ij_bounds, and the purpose of
c     doing it this way is to limit the number of points for which the
c     subroutine has to calculate distances (for a global 1 deg grid,
c     the number of loop iterations is reduced from 65160 to somewhere
c     around 600).
c
c     INPUT:
c     flon    Lon value for center point about which barnes anl is done
c     flat    Lat value for center point about which barnes anl is done
c     iimax   Max number of pts in x-direction on input grid
c     jjmax   Max number of pts in y-direction on input grid
c     iibeg   i index for grid point to start barnes anlysis (upp left)
c     jjbeg   j index for grid point to start barnes anlysis (upp left)
c     iiend   i index for last grid point in barnes anlysis (low right)
c     jjend   j index for last grid point in barnes anlysis (low right)
c     re      input e-folding radius for barnes analysis
c     ri      input influence radius for searching for min/max
c
c     OUTPUT:
c     favg    Average value about the point (flon,flat)
c     iret    Return code from this subroutine

      USE inparms; USE tracked_parms; USE trig_vals
      implicit none

      type (gficard) inp

      real         rlon(iimax),rlat(jjmax)
      real         res,wts,favg,dist,re,ri,flon,flat,wt
      integer      jjbeg,jjend,iibeg,iiend,i,j,icount,iret
      integer      iimax,jjmax
      character(*) cparm

c     --------------------------

      res  = re*re
      wts  = 0.0
      favg = 0.0

      icount = 0

      do j=jjbeg,jjend
        do i=iibeg,iiend

          icount = icount + 1

          call calcdist(flon,flat,xlon(i),ylat(j),dist)

          if (dist > ri) cycle

          wt   = exp(-1.0*dist*dist/res)
          wts  = wts + wt
          favg = favg + wt*xdata_2d(i,j)

        enddo
      enddo

      if (wts .gt. 1.0E-5) then
         favg = favg/wts
      else
         favg = 0.0
      endif
      iret = 0
c
      return
      end

c---------------------------------------------------------------------
c
c---------------------------------------------------------------------
      subroutine fix_latlon_to_ij (imax,jmax,grdspc,guesslon,guesslat
     &          ,ifix,jfix,inp,iunit_diag,callchar)
c
c     ABSTRACT: This subroutine takes an input lat/lon position and
c     assigns it to a nearby (i,j) gridpoint. 

      USE inparms; USE tracked_parms
      implicit none

      type (gficard) inp

      integer  imax,jmax,ifix,iunit_diag,jfix
      real     grdspc,guesslon,guesslat,glatmax,glonmin,xflon,yflat
      character callchar*1
c
      glatmax = ylat(1)
      glonmin = xlon(1)

c     Fix guesslat to the *nearest* j-point (i.e., round it....)

      if (guesslat >= 0.0) then    ! N. Hemisphere
        jfix = int((glatmax - guesslat)/grdspc + 1.0 + 0.5)
      else                        ! S. Hemisphere
        jfix = ceiling((glatmax - guesslat)/grdspc + 1.0 - 0.5)
      endif
 
c     Fix guesslon to the *nearest* i-point (i.e., round it....)

      ifix = int((guesslon - glonmin)/grdspc + 1.0 + 0.5)

c     Calculate the longitude and latitude of these ifix and
c     jfix points....

      xflon = glonmin + (ifix-1)*grdspc
      yflat = glatmax - (jfix-1)*grdspc

c      write (iunit_diag,111) guesslon,guesslat,callchar
c      write (iunit_diag,113) ifix,jfix,xflon,yflat
  111 format (1x,' fix_latlon:  guesslon= ',f8.3,'  guesslat= ',f8.3
     &       ,' Called from loc: ',a1)
  113 format (1x,' fix_latlon:  ifix= ',i4,' jfix= ',i4,'  fixlon= '
     &       ,f8.3,'  fixlat= ',f8.3)
c
      return
      end

c-----------------------------------------------------------------------
c
c-----------------------------------------------------------------------
      subroutine calcdist(rlonb,rlatb,rlonc,rlatc,xdist)
c
c     ABSTRACT: This subroutine computes the distance between two
c               lat/lon points by using spherical coordinates to
c               calculate the great circle distance between the points.
c                       Figure out the angle (a) between pt.B and pt.C,
c             N. Pole   then figure out how much of a % of a great
c               x       circle distance that angle represents.
c              / \
c            b/   \     cos(a) = (cos b)(cos c) + (sin b)(sin c)(cos A)
c            /     \
c        pt./<--A-->\c     NOTE: The latitude arguments passed to the
c        B /         \           subr are the actual lat vals, but in
c                     \          the calculation we use 90-lat.
c               a      \
c                       \pt.  NOTE: You may get strange results if you:
c                         C    (1) use positive values for SH lats AND
c                              you try computing distances across the
c                              equator, or (2) use lon values of 0 to
c                              -180 for WH lons AND you try computing
c                              distances across the 180E meridian.
c
c     NOTE: In the diagram above, (a) is the angle between pt. B and
c     pt. C (with pt. x as the vertex), and (A) is the difference in
c     longitude (in degrees, absolute value) between pt. B and pt. C.
c
c     !!! NOTE !!! -- THE PARAMETER ecircum IS DEFINED (AS OF THE
c     ORIGINAL WRITING OF THIS SYSTEM) IN KM, NOT M, SO BE AWARE THAT
c     THE DISTANCE RETURNED FROM THIS SUBROUTINE IS ALSO IN KM.
c
      USE trig_vals
      implicit none

      real   rlatb,rlatc,rlonb,rlonc,pole,difflon,distlatb,distlatc 
      real   cosanga,degrees,circ_fract,xdist
c
      if (rlatb < 0.0 .or. rlatc < 0.0) then
        pole = -90.
      else
        pole = 90.
      endif
c
      distlatb = (pole - rlatb) * dtr
      distlatc = (pole - rlatc) * dtr
      difflon  = abs( (rlonb - rlonc)*dtr )
c
      cosanga = ( cos(distlatb) * cos(distlatc) +
     &            sin(distlatb) * sin(distlatc) * cos(difflon))

c     This next check of cosanga is needed since I have had ACOS crash
c     when calculating the distance between 2 identical points (should
c     = 0), but the input for ACOS was just slightly over 1
c     (e.g., 1.00000000007), due to (I'm guessing) rounding errors.

      if (cosanga > 1.0) then
        cosanga = 1.0
      endif

      degrees    = acos(cosanga) / dtr
      circ_fract = degrees / 360.
      xdist      = circ_fract * ecircum
c
c     NOTE: whether this subroutine returns the value of the distance
c           in km or m depends on the scale of the parameter ecircum.
c           At the original writing of this subroutine (7/97), ecircum
c           was given in km.
c
      return
      end
