function main(inpargs)

*--------------------------------------------------------------------*
* Get input parameters
*
*  1st parameter: User's directory id (for printing)
*  2nd parameter: Storm name and atcf (used to open correct ctl file)
*  3rd parameter: Initialization time <yyyymmddhh> (also used to open ctl)
*  4th parameter: Forecast hour <hh> (used to open correct ctl and set correct time)
*  5th parameter: Determine which grid to open (1deg, 6thdeg or 12thdeg)
*  6th parameter: Starting latitude for linear xsection or center point latitude for
*                 azimuthal cross section
*  7th parameter: Starting longitude for linear xsection or center point longitude for 
*                 azimuthal cross section
*  8th parameter: Ending latitude for linear xsection or maximum range for azimuthal average
*  9th parameter: Ending longitude for linear xsection or starting and ending azimuth 
*                 for azimuthal average (example: 45 degrees to 135 degrees would be "045135")
*                 for full 360 degrees put "000360"
* 10th parameter: Variable to plot initially
* 11th parameter: Unit of initial variable (ignored if there's only one)
* 12th parameter: Type of cross section (linear, azimuthal, or simplevp (simple vertical 
*                 profile at a lat/lon point))

_userid = subwrd(inpargs, 1)
_stormid = subwrd(inpargs, 2)
_itime = subwrd(inpargs, 3)
_fhour = subwrd(inpargs, 4)
_nest = subwrd(inpargs, 5)
_lat1 = subwrd(inpargs, 6)
_lon1 = subwrd(inpargs, 7)
_lat2 = subwrd(inpargs, 8)
_lon2 = subwrd(inpargs, 9)
_var = subwrd(inpargs, 10)
_unit = subwrd(inpargs, 11)
_cstype = subwrd(inpargs, 12)

*-----------------------------------------------------------------------*
* Set default option values
*
* _botlev:     Lower most pressure level.
* _toplev:     Top most pressure level.
* _nrange:     Number of range sections to divide azimuthal average display into.
* _nazimuth:   Number of spokes to average in azimuthal average.
* _overlay:    New variables are ploted without erasing what's underneath.
* _shade:      Shade contour bands.
* _labels:     Contours are labeled.
* _zlog:       Vertical scale is logorithmic w.r.t. pressure.
* _drawground: Shade in gound using the surface pressure grid data.
* _gridvals:   Plot grid values on chart.
* _writedata:  For simple vertical profiles, this will write out the data to 
*              a sequentially formatted binary file if set to 'y'.
* _grdspcing:  Number of grit point values to plot per horizontal page unit.
* _grdsigdig:  Number of digits to display when plotting grid values.
* _vecspcing:  Number of vectors to plot per horizontal page unit.
* _vecscale:   Vector scaling factor to use when displaying wind vectors.
* _grddigsiz:  Size of text to use when plotting grid values.
* _stndrdschm: Use custom ".gs" script to set colors and contour levels. 
* _defcint:    Contour interval used if custom script isn't used.
* _defcclr:    Contour color (or rainbow) used if custom script isn't used.
* _nhlbls:     Number of horizontal labels to plot for azimuthal and arbitrary. 
*              x-sections (labelling is automatic for fixed x-sections).

_botlev     = 1000
_toplev     = 100
_nrange     = 61
_nazimuth   = 20
_overlay    = 'n'
_shade      = 'y'
_labels     = 'y'
_zlog       = 'y'
_drawground = 'n'
_gridvals   = 'n'
_writedata  = 'n'
_grdspcing  = 0.5
_grdsigdig  = 2
_grddigsiz  = 0.06
_vecspcing  = 0.25
_vecscale   = 50.0
_strmlnden  = 5
_stndrdschm = 'y'
_defcint    = 10
_defcclr    = "rainbow"
_nhlbls     = 11

*-----------------------------------------------------------------------*

* set unit option values

if (_var = 'u' | _var = 'v' | _var = "windmag")
  _windunit = _unit
else
  _windunit = "knots"
endif

if (_var = 't' | _var = "td" | _var = "tdev")
  _tempunit = _unit
else
  _tempunit = "degf"
endif

* determine whether cross section is a normal linear cross section or an azimuthal cross section

if (_cstype = "azimuthal")

  say 'cross section type = '_cstype
  _range = _lat2
  _startaz = substr(_lon2,1,3)
  _endaz   = substr(_lon2,4,3)

* do this so we don't count the northern point twice in the average  
  if (_startaz = 0 & _endaz = 360)
    _startaz = _startaz + 360 / _nazimuth
  endif
   
else

  if (_cstype = "simplevp")

    say 'cross section type = '_cstype
  
  else

    _cstype = "linear"
    say 'cross section type = '_cstype

  endif

endif

* Because grads will only pass 8 arguments to a user difined function we need an extra file
* to pass more arguments to a user define function

'!touch ./udfs/extended_udf_args.txt'

* set the virtual page bounds

_vpagelft = 0.0
_vpagergt = 10.5
_vpagebot = 0.2
_vpagetop = 7.5

'clear'

* set number of labels to zero (_nlabel controls vertical location of labels when overlaying multiple plots) 

_nlabel = 0

* load ctl file, draw the menu, set up grids to plot, plot initial variable,
* draw the ground (if option is on), and plot the title

rc = loadctl()

* here we need to check if the relative humidity is a fraction rather than a percent 
_rhbug = testrhbug()

rc = makemenu()
rc = setplot()
if (_var = "vecwind" | _var = "streamwind"); rc = plotvecstrm(); else; rc = plotvar(); endif
if (_drawground = 'y')
  if (_cstype != "azimuthal" & _cstype != "simplevp") 
    rc = plotground() 
  endif
endif
rc = plttitle()

*--------------------------------------------------------*
* Main loop starts here

while (1)
  
  'q pos'

  mousenum = subwrd(result, 5)
  widclass = subwrd(result, 6)
  button   = subwrd(result, 7)
  
  if (widclass = 3 & button = 1)
*   "Main" menu button selected

    item = subwrd(result, 8)
    choice = _mainmenu.item
    if (choice = "redraw")
*     user has selected to redraw the display, clearing any old plots

      if (_var != "none")
        'clear'
        _nlabel = 0
        rc = makemenu()
        rc = setplot()
        if (_var = "vecwind" | _var = "streamwind"); rc = plotvecstrm(); else; rc = plotvar(); endif
        if (_drawground = 'y') 
          if (_cstype != "azimuthal" & _cstype != "simplevp") 
            rc = plotground() 
          endif
        endif
        if (_gridvals = 'y'); rc = pltgrdvals(); endif
        rc = plttitle()
      endif
    endif
    if (choice = "printbw" | choice = "printcl" | choice = "printgif" | choice = "printrgif" | choice = "printpng" | choice = "printrpng")
*     user has selected to print to a b/w or color postcript file or to a gif file    
    
      _hardcopy = choice
      rc = output()
    endif
    if (choice = "return")
*     user has selected to exit cross section script but keep GrADS environment open

      return
    endif
    if (choice = "close")
*     user has selected to close the current GrADS window
    
      quit
    endif
  endif
  
  if (widclass = 3 & button = 2)
*   "Vars" menu button selected

    item = subwrd(result, 8)
    
* chack first character of selected variable to see if it's a valid

    char = substr(_varmenu.item,1,1)
    if (char = "" | char = "_")
      _var = "none"
    else
      _var = _varmenu.item
    endif

    if (_var != "none")
      if (_overlay = 'n')
        'clear'
        rc = makemenu()
      endif        
      rc = setplot()
      if (_var = "vecwind" | _var = "streamwind"); rc = plotvecstrm(); else; rc = plotvar(); endif
      if (_drawground = 'y') 
        if (_cstype != "azimuthal" & _cstype != "simplevp") 
          rc = plotground() 
        endif
      endif
      if (_gridvals = 'y'); rc = pltgrdvals(); endif
      rc = plttitle()
    endif
  endif

  if (widclass = 3 & button = 3)
*   "Options menu button selected

    item = subwrd(result, 8)
    item2 = subwrd(result, 10)
    item3 = subwrd(result, 12)
    rc = pickopt(item, item2, item3)
  endif
  
  if (widclass = 3 & button = 4)
*   "Units menu button selected

    item = subwrd(result, 8)
    item2 = subwrd(result, 10)
    rc = pickunit(item, item2)
  endif
  
endwhile

return

*-------------------------*
*
*-------------------------*
function setplot()

* This function sets up the grid data that will be plotted with the plotvar() function.

*-------------------------------------------------------------------------------------*
* The following code sets the bounds of grid area and determines the direction of the
* cross section."horizontal" means fixed latitude, "vertical" means fixed longitude,
* "xmajordiag" means diagonal with longitude range larger than latitude range, 
* and "ymajordiag" means diagonal with latitude range larger than longitude range.

* get north/south and east/west latitude and longitude bounds

  if (_cstype = "azimuthal")

* for an azimuthal average cross section we set the dimensions for the azimuthal disk's
* bounding rectange

    _nlat = _lat1 + _range
    _slat = _lat1 - _range
    _elon = _lon1 + _range / math_cos(_nlat*3.141592/180)
    _wlon = _lon1 - _range / math_cos(_nlat*3.141592/180)

    sbound = _slat - 0.2
    nbound = _nlat + 0.2
    wbound = _wlon - 0.2
    ebound = _elon + 0.2    

    'set lat '%sbound' '%nbound
    'set lon '%wbound' '%ebound
    'set lev '%_botlev%' '%_toplev

    _dir = "azimuthal"

  else

    if (_cstype = "simplevp")

      _nlat = _lat1
      _slat = _lat2
      _elon = _lon1
      _wlon = _lon2

      'set lat '_slat
      'set lon '_wlon
      'set lev '_botlev' '_toplev

      _dir = "point"

    else

      if(_lat1 > _lat2)
        _nlat = _lat1
        _slat = _lat2
      else
        _nlat = _lat2
        _slat = _lat1
      endif

      if(_lon1 > _lon2)
        _elon = _lon1
        _wlon = _lon2
      else
        _elon = _lon2
        _wlon = _lon1
      endif

      if (_slat = _nlat)

* the code below collects surface pressure data points so we can draw terrain
    
        if (_drawground = 'y')
          'set x 1'
          'set y 1'
          'set lev 1000'
          inc = (_elon - _wlon) / 100.0
          i = 0; lon = _wlon
          while (i <= 100)
            'set gxout stat'
            'd gr2stn(ps,'lon','_slat')'
            rec = sublin(result,8)
            _sfc.pres.i = subwrd(rec,5) / 100.0
            i = i+1
            lon = lon + inc
          endwhile
        endif
    
* now set the dimensions
    
        'set lat '_slat
        'set lon '%_wlon%' '%_elon
        'set lev '%_botlev%' '%_toplev
    
        _dir = "horizontal"
       
      else 

        if (_wlon = _elon)
  
* the code below collects surface pressure data points so we can draw terrain
  
          if (_drawground = 'y')
            'set x 1'
            'set y 1'
            'set lev 1000'
            inc = (_nlat - _slat) / 100.0
            i = 0; lat = _slat
            while (i <= 100)
              'set gxout stat'
              'd gr2stn(ps,'_wlon','lat')'
              rec = sublin(result,8)
              _sfc.pres.i = subwrd(rec,5) / 100.0
              i = i+1
              lat = lat + inc
            endwhile
          endif

* now set the dimensions

          'set lat '%_slat%' '%_nlat
          'set lon '_wlon
          'set lev '%_botlev%' '%_toplev
    
          _dir = "vertical"
    
        else
  
* for an arbitrary cross section we set the dimensions for the cross section's
* bounding rectangle

          sbound = _slat - 0.2
          nbound = _nlat + 0.2
          wbound = _wlon - 0.2
          ebound = _elon + 0.2
        
          'set lat '%sbound%' '%nbound
          'set lon '%wbound%' '%ebound
          'set lev '%_botlev%' '%_toplev
    
          londiff = _lon2 - _lon1
          latdiff = _lat2 - _lat1
    
          if ( math_abs(londiff) > math_abs(latdiff) )
            _dir = "xmajordiag"
          else
            _dir = "ymajordiag"
          endif

        endif
      
      endif 

    endif

  endif

*----------------------------------------------------------------------------------------*  
* The following code block defines a new gridded variable ('pvar') which will be plotted 
* in the plotvar() function.  The vector wind has two variables 'pvar' and 'pvar2'.
  
  if (_var = "windmag")
  
    'define pvar = mag(u,v)'

  else; if(_var = "radwind" | _var = "tanwind")

* Just assign pvar to pmsl for now. Tangential and Radial values will be calculated
* from cylindrical grids of u and v later

    'define pvar = pmsl'
              
  else; if ((_var = "vecwind" | _var = "streamwind"))

    if (_dir = "azimuthal")
    
* Just assign pvar to pmsl for now. Radial wind values will be calculated
* from cylindrical grids of u and v later.

      'define pvar = pmsl'
      
    else
    
      rc = cswndgrd()
      
    endif

  else; if (_var = "theta")
  
    rc = thtgrd()
    
  else; if (_var = "thetae")
    
    rc = thtegrd()
    
  else; if (_var = "td")
  
    rc = tdgrd()
    
  else; if (_var = "tdev")
  
    rc = tdevgrd()
    
  else
    
    'define pvar = '_var
        
  endif; endif; endif; endif; endif; endif; endif
      
*-----------------------------------------------------------------------------------------*
* This code creates a new grid (using the collect command) for an arbitrary cross section. 
* If the longitude range is larger than the latitude range, the x axis represents longitude.
* Otherwise the x axis represents latitude. In either case the original grid is interpolated 
* at 101 equally spaced points along the cross section line and this interpolated data 
* is stored as collection of 101 vertical profiles which are later used to display the cross
* section in  plotvar().
  
  if (_dir = "xmajordiag")
              
    'collect 1 free'
    'collect 2 free'
    'set x 1'
    'set y 1'
            
    lon = _lon1
    i = 0
    inc = (londiff) / 100.0
    while( i <= 100)
      
      lat = _lat1 + latdiff * (lon - _lon1) / londiff

      'collect 1 gr2stn(pvar,'lon','lat')'
      if (_var = "vecwind" | _var = "streamwind")
        'collect 2 gr2stn(pvar2,'lon','lat')'
      endif

* the code below collects surface pressure data points so we can draw terrain

      if (_drawground = 'y')
        'set lev 1000'
        'set gxout stat'
        'd gr2stn(ps,'lon','lat')'
        rec = sublin(result,8)
        _sfc.pres.i = subwrd(rec,5) / 100.0
        'set lev '%_botlev%' '%_toplev
      endif
      
* construct string to control tick mark labels and spacing

      if (math_mod(i,_nhlbls-1) = 0)
        label = formatlon(math_nint(lon*100.0)/100.0)
        if (i = 0)
          _ticklbls = label
        else
          _ticklbls = _ticklbls%" | "%label
        endif
      endif

      i = i+1
      lon = lon + inc
    endwhile
        
    'set x 1 101'
    _ticklbls = _ticklbls%' '
    
  endif
  
  if (_dir = "ymajordiag")
          
    'collect 1 free'
    'collect 2 free'
    'set x 1'
    'set y 1'
      
    lat = _lat1
    i = 0
    inc = (latdiff) / 100.0
    while( i <= 100)
       
      lon = _lon1 + londiff * (lat - _lat1) / latdiff

      'collect 1 gr2stn(pvar,'lon','lat')'
      if (_var = "vecwind" | _var = "streamwind")
        'collect 2 gr2stn(pvar2,'lon','lat')'
      endif

* the code below collects surface pressure data points so we can draw terrain

      if (_drawground = 'y')
        'set lev 1000'
        'set gxout stat'
        'd gr2stn(ps,'lon','lat')'
        rec = sublin(result,8)
        _sfc.pres.i = subwrd(rec,5) / 100.0
        'set lev '%_botlev%' '%_toplev
      endif
          
* construct string to control tick mark labels and spacing
      
      if (math_mod(i,_nhlbls-1) = 0)
        label = formatlat(math_nint(lat*100.0)/100.0)
        if (i = 0)
          _ticklbls = label
        else
          _ticklbls = _ticklbls%" | "%label
        endif
      endif
      
      i = i+1
      lat = lat + inc
    endwhile
        
    'set x 1 101'
    _ticklbls = _ticklbls%' '

  endif

*-----------------------------------------------------------------------------------------*
* Here we create cylindrical grid data files to display using the cyl2aza user defined
* function later on.

  if (_dir = "azimuthal")

    _cylpvarf  = "cylpvarf.dat"
    _cylpvar2f = "cylpvar2f.dat"
    cyluwndf  = "cyluwndf.dat"
    cylvwndf  = "cylvwndf.dat"
    cyldummyf  = "cyltwndf.dat"
    
    if (_var = "tanwind" | _var = "radwind" | _var = "vecwind" | _var = "streamwind")

      'define pvar = u'
      rc = azgrd(cyluwndf)

      'define pvar = v'
      rc = azgrd(cylvwndf)

      if (_var = "tanwind")
        rc = uv2tr(cyluwndf,cylvwndf,_cylpvarf,cyldummyf)
      endif

      if (_var = "radwind")
        rc = uv2tr(cyluwndf,cylvwndf,cyldummyf,_cylpvarf)
      endif
      
      if (_var = "vecwind")
        rc = uv2tr(cyluwndf,cylvwndf,cyldummyf,_cylpvarf)
        if( _zlog = 'y' & _var = "vecwind")
          'define pvar = omega / (lev * 100)'
        else
          'define pvar = omega'
        endif
        rc = azgrd(_cylpvar2f)
      endif
      
      if (_var = "streamwind")
        rc = uv2tr(cyluwndf,cylvwndf,cyldummyf,_cylpvarf)
        'define pvar = omega'
        rc = azgrd(_cylpvar2f)
      endif
      
      '!rm ./udfs/'cyluwndf
      '!rm ./udfs/'cylvwndf
      '!rm ./udfs/'cyldummyf
                  
    else

      rc = azgrd(_cylpvarf)

    endif

* construct string to control tick mark labels and spacing

    i = 0
    while (i <= _nhlbls-1)
      label = math_nint(i * _range * 111.177 / 10)
      label = label%"km"
      if (i=0)
        _ticklbls = label
      else
        _ticklbls = _ticklbls%" | "%label
      endif 
      i = i+1
    endwhile

   _ticklbls = _ticklbls%' '

* set dimensions for display
  
    'set x 1 '_nrange
    'set y 1'
    'set lev '_botlev' '_toplev

  endif      
    
return

*-------------------------*
*
*-------------------------*
function plotvar()

* This function sets up the display options and plots the current variable.

*---------------------------------------------------------------*
* If the cross section was arbitrary or azimuthal, we need to create
* a new grid using the collection created in setplot() before displaying.
* Otherwise the regular grid is used.

  if (_dir =  "xmajordiag" | _dir = "ymajordiag")

    _dispstr = "coll2gr(1,-u)"

  else; if(_dir = "azimuthal")

    _dispstr = "cyl2aza("%_cylpvarf%",pvar)"
    _drawground = 'n'

  else

    _dispstr = "pvar"

  endif; endif
    
*----------------------------------------------------------------*
* Depending on the units being used, the gridded data may need to be
* scaled and shifted before displaying. "_convfctr1" is the scale factor,
* and "_convfctr2" is the shift factor.

  convfctr1 = 1.0
  convfctr2 = 0.0
  
  if (_var = 'u' | _var = 'v' | _var = "tanwind" | _var = "radwind" | _var = "windmag")
  
    _unit = _windunit
    
    if (_unit = "knots")
      convfctr1 = 1.9427
      convfctr2 = 0.0
    else; if (_unit = "mph")
      convfctr1 = 2.2341
      convfctr2 = 0.0
    endif; endif
    
  else; if (_var = 't' | _var = "td")
  
    _unit = _tempunit
    
    if (_unit = "degf")
      convfctr1 = 9.0 / 5.0
      convfctr2 = -459.67
    else; if (_unit = "degc")
      convfctr1 = 1.0
      convfctr2 = -273.15
    endif; endif
            
  else; if (_var = "tdev")
  
    _unit = _tempunit
    
    if (_unit = "degf")
      convfctr1 = 9.0 / 5.0
      convfctr2 = 0.0
    endif

  endif; endif; endif
 
* relative humidity is multiplied by 100 if it's a fraction
  
  if (_var = "rh" & _rhbug = 1)
  
    _dispstr = _dispstr%"*100"
  
  endif
    
  _dispstr = _dispstr%"*"%convfctr1%"+"%convfctr2
  
*----------------------------------------------------------------------*
* This code sets various display options (based on user selected options)
* before calling the display command.
        
  'set vpage '_vpagelft' '_vpagergt' '_vpagebot' '_vpagetop
  'set grads off'
  
  if ( _zlog = 'y')
    'set zlog on'
  else
    'set zlog off'
  endif

  if (_shade = 'y')
    'set gxout shaded'
    _ptype = "shaded" 
  else 
    'set gxout contour'
    _ptype = "contour"
  endif
  
  if (_labels = 'y')
    'set clab on'
  else
    'set clab off'
  endif 
  
* here we look for a custom standard scheme script file for the selected variable
  
  schmscript = "set_"%_var%"_scheme.gs"

  checkfile="/net/"%_userid%"/temp/check_"%_var%"_scheme.out"
  '!/bin/ls -C1 'schmscript' | wc -l >'checkfile
  res = read(checkfile)
  rc = sublin(res,1)
  if (rc != 0)
    if (rc = 2)
      say 'End of checkfile file: 'checkfile
    endif
    if(rc = 1); say 'rc=1: OPEN ERROR FOR 'checkfile; endif
    if(rc = 8); say 'rc=8: 'checkfile' OPEN FOR WRITE ONLY'; endif
    if(rc = 9); say 'rc=9: I/O ERROR FOR 'checkfile; endif
    ncheck = 0
  else
    srec = sublin(res,2)
    ncheck = subwrd(srec,1)
  endif
  
  rc = close(checkfile)

* if the standard scheme option is set and the file is found, run it 
       
  if(_stndrdschm = 'y' & ncheck != 0)
    'run '%schmscript%' '%_ptype%' '%_unit
  else
    'set cint '_defcint
    'set ccolor '_defcclr
    if (_defcclr = "rainbow")
      'run setrbcols.gs'
    endif
  endif
  
* if this is an arbitrary cross section, set the correct x-axis labels
  
  if(_dir = "xmajordiag" | _dir = "ymajordiag" | _dir = "azimuthal")
    'set xlabs '_ticklbls
  endif
      
* draw the plot
      
  'display '_dispstr
  
* if contours are shaded draw a color key 
  
  if(_ptype = "shaded")
    'run cbarn_medium.gs 1.0 0 6.0 0.5'
  endif

* If this is a simple vertical profile plot and the user has requested 
* that the data be written out to a binary file, then write the data
* out now.

  if (_cstype = 'simplevp')
    if (_writedata = 'y')
      slatlbl = formatlat(math_nint(_slat * 100) / 100.0)
      wlonlbl = formatlon(math_nint(_wlon * 100) / 100.0)
      odir = '/net/'%_userid%'/hurplots'
      sqfile = odir%'/'%_stormid%'.'%_itime%'.f'%_fhour%'.'%_var%'.'%slatlbl%'.'%wlonlbl%'.vpdat'
*      sqfile = odir%'/test.dat'
      'set fwrite -sq 'sqfile
      'set gxout fwrite'
      'd '_dispstr
      'disable fwrite'
    endif
  endif
    
return

*-------------------------*
*
*-------------------------*
function plotvecstrm()

* This function sets up the display options and plots vector winds or streamlines.

  if (_var = "vecwind")
    _ptype = "vector"
  else
    _ptype = "streamline"
  endif
  
* we can't plot grid values for vector data
  
  _gridvals = 'n'
  
*---------------------------------------------------------------*
* If the cross section was arbitrary, we need to create new grids
* using the collections created in setplot() before displaying.
* Otherwise the regular grids are used.

  if (_dir =  "xmajordiag" | _dir = "ymajordiag")
    dispvar1 = "coll2gr(1,-u)"
    dispvar2 = "coll2gr(2,-u)"
  else; if (_dir = azimuthal)
    dispvar1 = "cyl2aza("%_cylpvarf%",pvar)"
    dispvar2 = "cyl2aza("%_cylpvar2f%",pvar)"
  else
    dispvar1 = "pvar"
    dispvar2 = "pvar2"
  endif; endif

*----------------------------------------------------------------*
* Now we calculate the scaling factors for the vertical and horizontal
* components of the vector wind.

* first we need to get the plot area dimensions

  'set vpage '_vpagelft' '_vpagergt' '_vpagebot' '_vpagetop
  'set grads off'
  'set arrlab off'
  if ( _zlog = 'y')
    'set zlog on'
  else
    'set zlog off'
  endif
  'set gxout contour'
  'set black -9999999 9999999'
  if(_dir = "xmajordiag" | _dir = "ymajordiag" | _dir = "azimuthal")
    'set xlabs '_ticklbls
  endif
  'd 'dispvar2

  'q gxinfo'
  rec = sublin(result,3)
  gxlft = subwrd(rec,4); gxrgt = subwrd(rec,6)
  rec = sublin(result,4)
  gxbot = subwrd(rec,4); gxtop = subwrd(rec,6)
  
  swidth = gxrgt - gxlft
  sheight = gxtop - gxbot

* calculate horizontal scale factor

  if ( _dir = "horizontal" | _dir = "vertical" | _dir = "point")
    dellon = _elon - _wlon
    dellat = _nlat - _slat
  endif
  
  if (_dir = "xmajordiag" | _dir = "ymajordiag")
    dellon = _lon2 - _lon1
    dellat = _lat2 - _lat1
  endif

  if (_dir = "azimuthal")
    xsclfctr = 1.0
  else
    midlat = (_nlat + _slat) * 0.5
    scldlon = math_cos(midlat*3.14159/180.0) * dellon
    radical = math_sqrt(scldlon*scldlon + dellat*dellat)
    xsclfctr = radical
  endif

* calculate vertical scale factor
  
  if (_dir = "azimuthal")
    if (_zlog = 'y' & _var = "vecwind")
      ysclfctr = (sheight/swidth) * (6370000*3.14159/180.0) * _range / math_log(_toplev/_botlev)
    else
      ysclfctr = (sheight/swidth) * (6370000*3.14159/180.0) * _range / ((_toplev - _botlev)*100)
    endif
  else
    if (_zlog = 'y' & _var = "vecwind")
      ysclfctr = (sheight/swidth) * (6370000*3.14159/180.0) * radical / math_log(_toplev/_botlev)
    else
      ysclfctr = (sheight/swidth) * (6370000*3.14159/180.0) * radical / ((_toplev - _botlev)*100)
    endif
  endif

  if (_var = "vecwind")

*----------------------------------------------------------------------*
* This code sets up various options in order to displays vectors
    
* if this is an arbitrary cross section, set the correct x-axis labels
  
    if(_dir = "xmajordiag" | _dir = "ymajordiag" | _dir = "azimuthal")
      'set xlabs '_ticklbls
    endif

* Get unit conversion factors for vectors.
  
    if (_windunit = "knots")
      convfctr = 1.9427
    else; if (_windunit = "mph")
      convfctr = 2.2341
    else
      convfctr = 1.0
    endif; endif
  
* calculate the vector skip interval

    if(_dir = "vertical")
      'q dims'
      rec = sublin(result,3)
      grdwidth = subwrd(rec,13) - subwrd(rec,11)
    else
      'q dims'
      rec = sublin(result,2)
      grdwidth = subwrd(rec,13) - subwrd(rec,11)
    endif

    skip = math_nint(grdwidth * _vecspcing / swidth)

* draw the vector plot

    'set gxout vector'
    'set arrscl 0.5 '_vecscale
    'display skip('dispvar1'*'xsclfctr'*'convfctr','skip',0);'dispvar2'*'ysclfctr'*'convfctr

* Now create an arrow in a box on the upper right hand side of the display.
* This serves as a legend for the arrow scale.

*draw the bounding box

    xl = gxrgt - 0.7
    xr = gxrgt
    yt = gxtop
    yb = gxtop - 0.5
 
    'set line 0 1 6'
    'draw polyf 'xr' 'yb' 'xl' 'yb' 'xl' 'yt' 'xr' 'yt' 'xr' 'yb
    'set line 1 1 6'
    'draw rec 'xl' 'yb' 'xr' 'yt
 
* draw the arrow
 
    arrtail = xl + 0.1
    arrtip = arrtail + 0.5
    arrhead = arrtip - 0.075
    arrymid = yt - 0.15
    arrytop = arrymid + 0.05
    arrybot = arrymid - 0.05

* draw the label

    'set line 1 1 4'
    'draw line 'arrtail' 'arrymid' 'arrtip' 'arrymid
    'draw line 'arrhead' 'arrytop' 'arrtip' 'arrymid
    'draw line 'arrtip' 'arrymid' 'arrhead' 'arrybot
 
    xmid = xl + 0.35
    ymid = yb + 0.2
 
    'set strsiz 0.1 0.1'
    'set string 1 c 4'
    'draw string 'xmid' 'ymid' '_vecscale
    
  else
  
*----------------------------------------------------------------------*
* This code sets up various options in order to displays streamlines.

* if this is an arbitrary cross section, set the correct x-axis labels
  
    if(_dir = "xmajordiag" | _dir = "ymajordiag" | _dir = "azimuthal")
      'set xlabs '_ticklbls
    endif

* draw the streamlines

     'set gxout stream'
     'set strmden '_strmlnden
     'display 'dispvar1'*'xsclfctr';'dispvar2'*'ysclfctr

  endif

return

*-------------------------*
*
*-------------------------*
function plotground()

* This function draws the ground level on top of the plotted cross section
* The scaling had to be done manually because the grads scaling conversions
* would not work with arbitrary cross sections

  'set rgb 99 90 90 60'
  
  'q gxinfo'
  rec = sublin(result,3)
  gxinfox1 = subwrd(rec,4)
  gxinfox2 = subwrd(rec,6)
  rec = sublin(result,4)
  gxinfoy1 = subwrd(rec,4)
  gxinfoy2 = subwrd(rec,6)
   
  xrange = gxinfox2 - gxinfox1
  yrange = gxinfoy2 - gxinfoy1
  
  'q dims'
  rec = sublin(result,4)
  lolev = subwrd(rec,6)
  hilev = subwrd(rec,8)
  
  if (_zlog = 'y')
    h = yrange / math_log( hilev / lolev )
  else
    h = yrange / (lolev - hilev)
  endif
  
  polystr = gxinfox1%" "%gxinfoy1 
    
  px = gxinfox1
  i = 0
  inc = xrange / 100.0
     
  while (i <= 100)
  
    if (_zlog = 'y')
      py = gxinfoy1 + h * math_log( _sfc.pres.i / lolev )
    else
      py = gxinfoy1 + h * (lolev - _sfc.pres.i)
    endif
    
    if (py < gxinfoy1); py = gxinfoy1; endif
    
    polystr = polystr%" "%px%" "%py
    
    i = i+1
    px = px + inc
    
  endwhile
  
  polystr = polystr%" "%gxinfox2%" "%gxinfoy1
  polystr = polystr%" "%gxinfox1%" "%gxinfoy1
  
  'set line 99 1 1'
  'draw polyf 'polystr
      
return

*-------------------------*
*
*-------------------------*
function pltgrdvals()

  'q gxinfo'
  rec = sublin(result,3)
  swidth = subwrd(rec,6) - subwrd(rec,4)

  if(_dir = "vertical")
    'q dims'
    rec = sublin(result,3)
    grdwidth = subwrd(rec,13) - subwrd(rec,11)
  else
    'q dims'
    rec = sublin(result,2)
    grdwidth = subwrd(rec,13) - subwrd(rec,11)
  endif      

  skip = math_nint(grdwidth * _grdspcing / swidth)

  'set gridln off'
  'set gxout grid'
  'set dignum '_grdsigdig
  'set digsiz '_grddigsiz
  'display skip('_dispstr','skip',0)'

return

*-------------------------*
*
*-------------------------*
function plttitle()

* This function plots the title and labels on the top of the display

  'set vpage off'
  'set dfile 1'
  
* these variables control the height and width of the text characters
    
  hsiz = 0.15
  vsiz = 0.15
    
* get model verification time
    
  'q dims'
  rec = sublin(result, 5)
  _vtime = subwrd(rec, 6)
  _vtime = convtime(_vtime)

* set up label for plotted variable

  if (_var = "u")
    _varlabel = "u-component ("%_ptype%", "%_windunit%")"
  else; if (_var = "v")
    _varlabel = "v-component ("%_ptype%", "%_windunit%")"
  else; if (_var = "t")
    _varlabel = "temperature ("%_ptype%", "%_tempunit%")"
  else; if (_var = "hgt")
    _varlabel = "geopotential height ("%_ptype%", m)"
  else; if (_var = "rh")
    _varlabel = "relative humidity ("%_ptype%", %)"
  else; if (_var = "wvmix")
    _varlabel = "wv mixing ratio ("%_ptype%", g/kg)"
  else; if (_var = "cwmix")
    _varlabel = "cw mixing ratio ("%_ptype%", g/kg)"
  else; if (_var = "rwmix")
    _varlabel = "rw mixing ratio ("%_ptype%", g/kg)"
  else; if (_var = "icmix")
    _varlabel = "ice mixing ratio ("%_ptype%", g/kg)"
  else; if (_var = "omega")
    _varlabel = "vertical motion ("%_ptype%", pa/sec)"
  else; if (_var = "hrate")
    _varlabel = "diabatic heating ("%_ptype%", degC/dy)"
  else; if (_var = "cheat")
    _varlabel = "conv. heating ("%_ptype%", degC/dy)"
  else; if (_var = "lheat")
    _varlabel = "lg. scl. heating ("%_ptype%", degC/dy)"
  else; if (_var = "windmag")
    _varlabel = "wind magnitude ("%_ptype%", "%_windunit%")"
  else; if (_var = "tanwind")
    _varlabel = "tangential wind ("%_ptype%", "%_windunit%")"
  else; if (_var = "radwind")
    _varlabel = "radial wind ("%_ptype%", "%_windunit%")"
  else; if (_var = "vecwind")
    _varlabel = "wind circulation ("%_ptype%", "%_windunit%")"
  else; if (_var = "streamwind")
    _varlabel = "wind circulation ("%_ptype%")"
  else; if (_var = "theta")
    _varlabel = "potential temperature ("%_ptype%", K)"
  else; if (_var = "thetae")
    _varlabel = "theta-e ("%_ptype%", K)"
  else; if (_var = "td")
    _varlabel = "dewpoint ("%_ptype%", "%_tempunit%")"
  else; if (_var = "tdev")
    _varlabel = "temperature deviation ("%_ptype%", "%_tempunit%")"
  else
    _varlabel = "undefined"
  endif; endif; endif; endif; endif; endif; endif; endif
  endif; endif; endif; endif; endif; endif; endif; endif
  endif; endif; endif; endif; endif; endif
  
* put in storm id and grid nest text on upper left
  
  xstart = 0.5
  ystart = 8.0
  cstr = "GFDL forecast for "%_stormid%" "%_nest%" grid"
  'set string 1 l 4 0'
  'set strsiz 'hsiz' 'vsiz
  'draw string 'xstart' 'ystart' 'cstr
  
* go to next line and put in initialization and verification time
  
  ystart = ystart - vsiz*2
  cstr = "it: "%_itime%" vt: "%_vtime%" ("%_fhour%"h)"
  'set string 1 l 4 0'
  'set strsiz 'hsiz' 'vsiz
  'draw string 'xstart' 'ystart' 'cstr
  
* go to next line and put in cross section location
     
  ystart = ystart - vsiz*2
  if(_dir = "horizontal")
    slatlbl = formatlat(math_nint(_slat * 100) / 100.0)
    cstr = "x-sect: E-W at "%slatlbl
  else; if (_dir = "vertical")
    wlonlbl = formatlon(math_nint(_wlon * 100) / 100.0)
    cstr = "x-sect: N-S at "%wlonlbl
  else; if (_dir = "point")
    wlonlbl = formatlon(math_nint(_wlon * 100) / 100.0)
    slatlbl = formatlat(math_nint(_slat * 100) / 100.0)
    cstr = "Vertical profile at "%slatlbl%" "%wlonlbl
  else; if (_dir = "azimuthal")
    lat1lbl = formatlat(math_nint(_lat1 * 100) / 100.0)
    lon1lbl = formatlon(math_nint(_lon1 * 100) / 100.0)
    az1lbl  = substr(_lon2,1,3)
    az2lbl  = substr(_lon2,4,3)
    cstr = "azim. ave: "%az1lbl%"-"%az2lbl%" deg at "%lat1lbl%"; "%lon1lbl
  else
    lat1lbl = formatlat(math_nint(_lat1 * 100) / 100.0)
    lat2lbl = formatlat(math_nint(_lat2 * 100) / 100.0)
    lon1lbl = formatlon(math_nint(_lon1 * 100) / 100.0)
    lon2lbl = formatlon(math_nint(_lon2 * 100) / 100.0)
    cstr = "x-sect: "%lat1lbl%"; "%lon1lbl%" to "%lat2lbl%"; "%lon2lbl
  endif; endif; endif; endif
  
  'set string 1 l 4 0'
  'set strsiz 'hsiz' 'vsiz
  'draw string 'xstart' 'ystart' 'cstr
  
* set _nlabel to 0 if we're not overlaying plots ('_nlabel' controls how far down the 
* variable label is drawn)
  
  if( _overlay = 'n')
    _nlabel = 0
  endif
  
* draw in the current variable label
  
  xstart = 10.5
  ystart = 8.0
  ystart = ystart - vsiz * _nlabel * 2
  cstr = _varlabel
  'set string 1 r 4 0'
  'set strsiz 'hsiz' 'vsiz
  'draw string 'xstart' 'ystart' 'cstr
  
* increment '_nlabel' so that a new variable labels will not be drawn on top of each 
* other when overlaying
  
  _nlabel = _nlabel + 1
  
* plot GFDL label on lower right

  cstr = "GFDL Hurricane Dynamics Group"
  'set string 1 r 3 0'
  'set strsiz 0.09 0.09'
  'draw string 10.95 0.1 'cstr  

return

*-------------------------*
*
*-------------------------*
function azgrd(cylgrdf)

* This function creates a file 'cylpvarf.dat 'consisting of grid data interpolated into
* concentric rings for azimuthal averaging.

  say 'Creating cylindrical grid file 'cylgrdf'...'

* create temporary file to store grid data at multiple levels

  tempgrdf = "tempgrdf.dat"
        
* get number of vertical levels

  'q dims'
  rec = sublin(result,4)
  zbot = subwrd(rec,11)
  ztop = subwrd(rec,13)
  nlevs = ztop - zbot + 1

* We have to write the grid data out one level at a time to a customized file 
* because for some reason the GrADS user defined function feature doesn't support
* z varying data in define statements (don't ask why).

  z = zbot
  while(z <= ztop)
    'set z 'z
    'define dummy = apndgrd('tempgrdf',pvar)'
    'undefine dummy'
    z = z+1
  endwhile
 
* Here we write extra arguments into text file to be used by next user defined function call.
* It's a work-around to overcome the fact that user defined functions can't have more than
* 8 arguments.
    
  temp = math_format("%20.10f",_lat1)
  rc = write("./udfs/extended_udf_args.txt",temp)
  if (sublin(rc,1) != 0)
    say 'error writing arguments to extended udf arguments file'
    'quit'
  endif
  temp = math_format("%20.10f",_lon1)
  rc = write("./udfs/extended_udf_args.txt",temp,append)
  if (sublin(rc,1) != 0)
    say 'error writing arguments to extended udf arguments file'
    'quit'
  endif
  temp = math_format("%20.10f",_nazimuth)
  rc = write("./udfs/extended_udf_args.txt",temp,append)
  if (sublin(rc,1) != 0)
    say 'error writing arguments to extended udf arguments file'
    'quit'
  endif
  temp = math_format("%20.10f",_nrange)
  rc = write("./udfs/extended_udf_args.txt",temp,append)
  if (sublin(rc,1) != 0)
    say 'error writing arguments to extended udf arguments file'
    'quit'
  endif
  temp = math_format("%20.10f",nlevs)
  rc = write("./udfs/extended_udf_args.txt",temp,append)
  if (sublin(rc,1) != 0)
    say 'error writing arguments to extended udf arguments file'
    'quit'
  endif
  temp = math_format("%20.10f",_startaz)
  rc = write("./udfs/extended_udf_args.txt",temp,append)
  if (sublin(rc,1) != 0)
    say 'error writing arguments to extended udf arguments file'
    'quit'
  endif
  temp = math_format("%20.10f",_endaz)
  rc = write("./udfs/extended_udf_args.txt",temp,append)
  if (sublin(rc,1) != 0)
    say 'error writing arguments to extended udf arguments file'
    'quit'
  endif
  temp = math_format("%20.10f",_range)
  rc = write("./udfs/extended_udf_args.txt",temp,append)
  if (sublin(rc,1) != 0)
    say 'error writing arguments to extended udf arguments file'
    'quit'
  endif
  temp = math_format("%20.10f",_botlev)
  rc = write("./udfs/extended_udf_args.txt",temp,append)
  if (sublin(rc,1) != 0)
    say 'error writing arguments to extended udf arguments file'
    'quit'
  endif
  temp = math_format("%20.10f",_toplev)
  rc = write("./udfs/extended_udf_args.txt",temp,append)
  if (sublin(rc,1) != 0)
    say 'error writing arguments to extended udf arguments file'
    'quit'
  endif

  rc = close("./udfs/extended_udf_args.txt")

* For some odd reason a grads command needs to be called **after** the 'undefine' command
* and **before** the usere defined function is called.  Otherwise there is an error.
* Any command will work.

  'set grads off'

* now interpolate rectangular grid file to cylindrical grid file

  'define dummy = rec2cyl('tempgrdf','cylgrdf',pvar)'
  'undefine dummy'
  '!rm ./udfs/'tempgrdf

  'set z 'zbot' 'ztop

return

*-------------------------*
*
*-------------------------*
function uv2tr(ufile,vfile,tfile,rfile)

* This function converts cylindrical grid files of u and v wind components
* to cylindrical grid files of tangential and radial wind components

  say 'Calculating radial and tangential wind fields from u and v fields...'

  'define dummy = uv2tr('ufile','vfile','tfile','rfile',pvar)'
  'undefine dummy'

return

*-------------------------*
*
*-------------------------*
function cswndgrd()

* This function defines new grids for the horizontal and vertical
* components of the wind projected along the cross section.

  say "Calculating horizontal and vertical wind components..."

  if ( _dir = "horizontal" | _dir = "vertical")
    dellon = _elon - _wlon
    dellat = _nlat - _slat
  else
    dellon = _lon2 - _lon1
    dellat = _lat2 - _lat1
  endif

*  note: scldlon = scaled longitude

  'define scldlon = cos(lat*3.14159/180.0) * 'dellon
  'define pvar  = (u*scldlon + v*'dellat')/(scldlon*scldlon + 'dellat'*'dellat')'
  
  if( _zlog = 'y' & _var = "vecwind")
    'define pvar2 = omega / (lev * 100)'
  else
    'define pvar2 = omega'
  endif

  'undefine scldlon'

return

*-------------------------*
*
*-------------------------*
function thtgrd()

* This function defines a new grid for potential temperature 

  say "Calculating theta..."

  if (_mthermvals = 'y')
    'define pvar = t*pow(1000.0/lev,0.2854*(1.0 - 0.00028*wvmix))'
  else
    'define pvar = t*pow(1000.0/lev,0.2854)'
  endif

return

*-------------------------*
*
*-------------------------*
function thtegrd()

* This function defines a new grid for theta-e. The maskout function had to be
* used because some small mixing ratios are rounded to zero, which gives an error
* when the natural logorithm is taken. Small mixing ratios are set to .001.

  say "Calculating theta-e..."

  r = "(const(maskout(wvmix, wvmix - 0.001), 0.001, -u))" 
  'define e = lev * 'r' / (622 + 'r')'
  
  'define tl = 2840.0/(3.5*log(t) - log(e) - 4.805) + 55'
  'define pvar = t*pow(1000.0/lev,0.2854*(1.0 - 0.00028*'r')) * exp((3.376/tl - 0.00254)*'r'*(1+0.00081*'r'))' 

  'undefine tl'
  'undefine e'

return

*-------------------------*
*
*-------------------------*
function tdgrd()

* This function defines a new grid for dewpoint. The maskout function had to be
* used because some small mixing ratios are rounded to zero, which gives a 
* zero vapor pressure, which in turn causes calculation errors.

  say "Calculating dewpoints..."

  r = "(const(maskout(wvmix, wvmix - 0.001), 0.001, -u))"
    
*  'define es = 6.112 * exp(17.67*(t-273.15) / ((t-273.15) + 243.5))'
  
  'define e = (lev*'r')/(622.0+'r')'
  'define pvar = 243.5*log(e/6.108) / (17.67 - log(e/6.108)) + 273.15'
  
  'undefine e'
    
return

*-------------------------*
*
*-------------------------*
function tdevgrd()

  say "Calculating temperature deviations..."
   
  'define pvar = t - aave(t, lon='_wlon', lon='_elon', lat='_slat', lat='_nlat')'
  
return

*-------------------------*
*
*-------------------------*
function pickopt(item, item2, item3)
  
* This function controls what happens when the user selects from the options menu.
* 'item1' is the index of the item selected from the first drowdown menu, item2 is
* the index of the item selected from a submenu, and item3 is the index of the 
* item selected from a submenu of the first submenu.
  
  choice = _optmenu.item
  
  if (choice = "setlevs")
        
    say ' '
    say ' *-------------------------------------------*'
    say ' *                                           *'
    say ' *      ---- SET PRESSURE BOUNDS ----        *'
    say ' *                                           *'
    say ' * Enter the lower and upper pressure levels *'
    say ' * for the cross section (must be a multiple *'
    say ' * of 50)                                    *'
    say ' *                                           *'
    say ' *-------------------------------------------*'
    say ' '

    prompt 'Lower Level ---> '
    pull _botlev
    prompt 'Upper Level ---> '
    pull _toplev
            
  endif
  
  if (choice = "scheme")
  
    choice2 = _schmmenu.item2
    
    if (choice2 = "standard")
      if (_stndrdschm = 'y')
        _stndrdschm = 'n'
      else
        _stndrdschm = 'y'
      endif
    endif
    
    if (choice2 = "setcint")
      say ' '
      say ' *--------------------------------------------*'
      say ' *                                            *'
      say ' *     ----  SET CONTOUR INTERVAL  ----       *'
      say ' *                                            *'
      say ' * At the prompt below, enter the new contour *'
      say ' * interval.  Current contour interval = '_defcint
      say ' *                                            *'
      say ' *--------------------------------------------*'
      say ' '
      prompt ' ---> '

      pull _defcint

      say ' '
      say ' ++ New contour interval is '_defcint' ....'
      say ' '
      
      _stndrdschm = 'n'
      
    endif
   
    if (choice2 = "setcolor")
    
      choice3 = _colmenu.item3
      _defcclr = choice3
               
      _stndrdschm = 'n'
      
    endif
              
  endif
  
  if (choice = "overlay")
  
    if (_overlay = 'y')
      _overlay = 'n'
    else
      _overlay = 'y'
    endif
    
  endif
  
  if (choice = "shade")
    
    if (_shade = 'y')
      _shade = 'n'
    else
      _shade = 'y'
    endif
        
  endif
  
  if (choice = "labels")
  
    if (_labels = 'y')
      _labels = 'n'
    else
      _labels = 'y'
    endif
  
  endif
  
  if (choice = "zlog")
  
    if (_zlog = 'y')
      _zlog = 'n'
    else
      _zlog = 'y'
    endif
    
  endif
  
  if (choice = "drawground")

    if (_drawground = 'y')
      _drawground = 'n'
    else
      _drawground = 'y'
    endif

  endif
  
  if (choice = "gridvals")

    if (_gridvals = 'y')
      _gridvals = 'n'
    else
      _gridvals = 'y'
    endif

  endif

  if (choice = "writedata")

    if (_writedata = 'y')
      _writedata = 'n'
    else
      _writedata = 'y'
    endif

  endif

  if (choice = "gridopts")

    choice2 = _gopmenu.item2

    if (choice2 = "grddispspcing")
      say ' '
      say ' *--------------------------------------------*'
      say ' *                                            *'
      say ' *   ----  SET GRIDDED VALUES SPACING  ----   *'
      say ' *                                            *'
      say ' * At the prompt below, enter the spacing     *'
      say ' * (in inches) to use when plotting grid      *'
      say ' * values                                     *'
      say ' *                                            *'
      say ' * Just hitting return leaves it alone        *'
      say ' * alone.                                     *'
      say ' *                                            *'
      say ' * Currently, the grid data values spacing    *'
      say ' * is set to '_grdspcing
      say ' *                                            *'
      say ' *--------------------------------------------*'
      say ' '
      prompt ' ---> '

      pull string

      if (string = '')
        _grdspcing = _grdspcing
      else
        _grdspcing = string
      endif
    endif
  
    if (choice2 = "grddispdigits")
      say ' '
      say ' *--------------------------------------------*'
      say ' *                                            *'
      say ' *       ----  SET GRIDDED VALUES  ----       *'
      say ' *       ----  SIGNIFICANT DIGITS  ----       *'
      say ' *                                            *'
      say ' * At the prompt below, enter the number of   *'
      say ' * significant digits to display when         *' 
      say ' * plotting grid values.                      *'
      say ' *                                            *'
      say ' * Just hitting return leaves it alone        *'
      say ' * alone.                                     *'
      say ' *                                            *'
      say ' * Currently, the number of significant       *'
      say ' * digits is set to '_grdsigdig
      say ' *                                            *'
      say ' *--------------------------------------------*'
      say ' '
      prompt ' ---> '

      pull string

      if (string = '')
        _grdsigdig = _grdsigdig
      else
        _grdsigdig = string
      endif
    endif

    if (choice2 = "grddispsize")
      say ' '
      say ' *--------------------------------------------*'
      say ' *                                            *'
      say ' *     ----  SET GRID DISPLAY SIZE ----       *'
      say ' *                                            *'
      say ' * At the prompt below, enter the size you    *' 
      say ' * want data to display at when plotting      *' 
      say ' * grid values.                               *'
      say ' *                                            *'
      say ' * Just hitting return leaves it alone        *'
      say ' * alone.                                     *'
      say ' *                                            *'
      say ' * Currently, the grid display size is        *'
      say ' * set to '_grddigsiz
      say ' *                                            *'
      say ' *--------------------------------------------*'
      say ' '
      prompt ' ---> '

      pull string

      if (string = '')
        _grddigsiz = _grddigsiz
      else
        _grddigsiz = string
      endif
    endif

  endif

  if (choice = "vectopts")

    choice2 = _vopmenu.item2

    if (choice2 = "vecsize")
      say ' '
      say ' *--------------------------------------------*'
      say ' *                                            *'
      say ' *        ----  SET VECTOR SIZE ----          *'
      say ' *                                            *'
      say ' * At the prompt below, enter the HORIZONTAL  *' 
      say ' * wind magnitude that is represented by an   *' 
      say ' * arrow 1/2 inch long.                       *'
      say ' *                                            *'
      say ' * Just hitting return leaves it alone        *'
      say ' * alone.                                     *'
      say ' *                                            *'
      say ' * Currently, an arrow of length 1/2 is       *'
      say ' * scaled to '_vecscale' '_windunit
      say ' *                                            *'
      say ' *--------------------------------------------*'
      say ' '
      prompt ' ---> '

      pull string

      if (string = '')
        _vecscale = _vecscale
      else
        _vecscale = string
      endif
    endif
    
    if (choice2 = "vecspcing")
      say ' '
      say ' *--------------------------------------------*'
      say ' *                                            *'
      say ' *      ----  SET VECTOR SPACING  ----        *'
      say ' *                                            *'
      say ' * At the prompt below, enter the new vector  *'
      say ' * spacing value (in inches).                 *'
      say ' *                                            *'
      say ' * Just hitting return leaves it alone.       *'
      say ' *                                            *'
      say ' * Currently, the vector spacing value is     *'
      say ' * set to '_vecspcing
      say ' *                                            *'
      say ' *--------------------------------------------*'
      say ' '
      prompt ' ---> '

      pull string

      if (string = '')
        _vecspcing = _vecspcing
      else
        _vecspcing = string
      endif
    endif
  
  endif

  if (choice = "azaveopts")

    choice2 = _azopmenu.item2

    if (choice2 = "nazimuth")
      say ' '
      say ' *--------------------------------------------*'
      say ' *                                            *'
      say ' *  ----  SET # OF SPOKES TO AVERAGE ----     *'
      say ' *                                            *'
      say ' * At the prompt below, enter the number of   *'
      say ' * radial spokes you want the sript to        *'
      say ' * average for szimuthal displays.            *'
      say ' *                                            *'
      say ' * (note: entering a large number may cause   *'
      say ' * the program to churn for a few seconds     *'
      say ' * before displaying)                         *'
      say ' *                                            *'
      say ' * Just hitting return leaves it alone.       *'
      say ' *                                            *'
      say ' * Currently, the number of spokes is         *'
      say ' * set to '_nazimuth
      say ' *                                            *'
      say ' *--------------------------------------------*'
      say ' '
      prompt ' ---> '

      pull string

      if (string = '')
        _nazimuth = _nazimuth
      else
        _nazimuth = string
      endif
    endif

    if (choice2 = "nrange")
      say ' '
      say ' *--------------------------------------------*'
      say ' *                                            *'
      say ' *     ----  SET RADIAL RESOLUTION ----       *'
      say ' *                                            *'
      say ' * At the prompt below, enter the number of   *'
      say ' * radial rings you want the sript to divide  *'
      say ' * the cross section into szimuthal displays  *'
      say ' *                                            *'
      say ' * (note: entering a large number may cause   *'
      say ' * the program to churn for a few seconds     *'
      say ' * before displaying)                         *'
      say ' *                                            *'
      say ' * Just hitting return leaves it alone.       *'
      say ' *                                            *'
      say ' * Currently, the radial resolution is        *'
      say ' * set to '_nrange
      say ' *                                            *'
      say ' *--------------------------------------------*'
      say ' '
      prompt ' ---> '

      pull string

      if (string = '')
        _nrange = _nrange
      else
        _nrange = string
      endif
   endif

  endif
  
  if (choice = "strmlnden")

    say ' '
    say ' *--------------------------------------------*'
    say ' *                                            *'
    say ' *     ----  SET STREAMLINE DENSITY ----      *'
    say ' *                                            *'
    say ' * At the prompt below, enter the new         *'
    say ' * streamline density value (a value from 1   *'
    say ' * to 10)                                     *'
    say ' *                                            *'
    say ' * Just hitting return leaves it alone.       *'
    say ' *                                            *'
    say ' * Currently, the streamline density is       *'
    say ' * set to '_strmlnden
    say ' *                                            *'
    say ' *--------------------------------------------*'
    say ' '
    prompt ' ---> '

    pull string

    if (string = '')
      _strmlnden = _strmlnden
    else
      _strmlnden = string
    endif

  endif
  
  rc = makeoptmenu()
    
return

*-------------------------*
*
*-------------------------*
function pickunit(item, item2)

  choice = _unitmenu.item
  
  if (choice = "temp")
  
    choice2 = _tempmenu.item2
    _tempunit = choice2
            
  endif
  
  if (choice = "wind")
    
    choice2 = _windmenu.item2
    _windunit = choice2
        
  endif

  rc = makeunitmenu()

return

*-------------------------*
*
*-------------------------*
function makemenu()

  rc = makemainmenu()
  rc = makevarmenu()
  rc = makeoptmenu()
  rc = makeunitmenu()

return

*-------------------------*
*
*-------------------------*
function makemainmenu()

  menustring  = "Redraw Display | Print B&W Postscript | Print Color Postscript | Print Color GIF | Print Reverse Color GIF | Print Color PNG | Print Reverse Color PNG | Exit Script | Close Window "
  _mainmenu.1 = "redraw"
  _mainmenu.2 = "printbw"
  _mainmenu.3 = "printcl"
  _mainmenu.4 = "printgif"
  _mainmenu.5 = "printrgif"
  _mainmenu.6 = "printpng"
  _mainmenu.7 = "printrpng"
  _mainmenu.8 = "return"
  _mainmenu.9 = "close"
  
  'clear dropmenu 1'
  'draw dropmenu 1 1.00 8.35 1.40 0.25 Main | 'menustring

return 

*-------------------------*
*
*-------------------------*
function makevarmenu()

  menustring = ""
  
  'q file'
  rec = sublin(result, 6)
  numvar = subwrd(rec, 5)
  
  if (numvar = 0)
    menustring = "No Variables Loaded"
    return 99
  else
  
    got_u = 'n'
    got_v = 'n'
    got_t = 'n'
    got_rh = 'n'
    _got_r = 'n'
    _got_pp = 'n'
    
    itmp = 1
    Ltmp = 7
    
    while (itmp <= numvar)
    
      tmplin = sublin(result, Ltmp)
      _varmenu.itmp = subwrd(tmplin, 1)
      
      if (itmp = 1)
        menustring = menustring%_varmenu.itmp
      else
        menustring = menustring%" | "%_varmenu.itmp
      endif

      if (_varmenu.itmp = 'u' | _varmenu.itmp ='U')
        got_u = 'y'
      endif

      if (_varmenu.itmp = 'v' | _varmenu.itmp = 'V')
        got_v = 'y'
      endif

      if (_varmenu.itmp = 't' | _varmenu.itmp = 'T')
        got_t = 'y'
      endif

      if (_varmenu.itmp = 'wvmix' | _varmenu.itmp = 'WVMIX')
        got_wvmix = 'y'
      endif

      if (_varmenu.itmp = 'ps' | _varmenu.itmp = 'PS')
        got_ps = 'y'
      endif

      if (_varmenu.itmp = 'omega' | _varmenu.itmp = 'OMEGA')
        got_omega = 'y'
      endif

      itmp = itmp + 1
      Ltmp = Ltmp + 1
      
    endwhile
    
    if ( got_ps != 'y')
      _drawground = 'n'
    endif
    
    _var.itmp = ""
    menustring = menustring%" | ---- Derived ----"
    itmp = itmp + 1
    
* add derived variables here


    if (got_u = 'y' & got_v = 'y')
          
      _varmenu.itmp = "windmag"
      itmp = itmp + 1
      
      menustring = menustring%" | Wind Magnitude"
      
      if (_cstype = "azimuthal")
      
        _varmenu.itmp = "radwind"
        itmp = itmp + 1

        menustring = menustring%" | Radial Wind"

        _varmenu.itmp = "tanwind"
        itmp = itmp + 1

        menustring = menustring%" | Tangential Wind"

      endif
            
    endif
    
    if (got_u = 'y' & got_v = 'y' & got_omega = 'y')

      _varmenu.itmp = "vecwind"
      itmp = itmp + 1

      menustring = menustring%" | Wind Circulation"

      _varmenu.itmp = "streamwind"
      itmp = itmp + 1
      
      menustring = menustring%" | Wind Streamlines"

    endif
    
    if (got_t = 'y')
    
      _varmenu.itmp = "theta"
      itmp = itmp + 1
      
      menustring = menustring%" | Theta"
      
      _varmenu.itmp = "tdev"
      itmp = itmp + 1
      
      menustring = menustring%" | Temp Deviation"
      
    endif
    
    if (got_wvmix = 'y' & got_t = 'y')
      
      _mthermvals = 'y'
      
      _varmenu.itmp = "thetae"
      itmp = itmp + 1
      
      menustring = menustring%" | Theta-e"
      
      _varmenu.itmp = "td"
      itmp = itmp + 1
      
      menustring = menustring%" | Dewpoint"
      
    endif
           
  endif
      
  menustring = menustring%' '
  
  'clear dropmenu 2'
  'draw dropmenu 2 2.40 8.35 0.80 0.25 Vars | 'menustring
    
return

*------------------------*
*
*------------------------*
function makeoptmenu()
  
  optstring = "Set Levels | Set Scheme >31> | Overlay Plots(toggle) "
  if (_overlay = 'y')
    optstring = optstring%"* "
  endif
  optstring = optstring%"| Shaded Contours(toggle) "
  if (_shade = 'y')
    optstring = optstring%"* "
  endif
  optstring = optstring%"| Contour Labels(toggle) "
  if (_labels = 'y')
    optstring = optstring%"* "
  endif
  optstring = optstring%"| Log-P Scale(toggle) "
  if (_zlog = 'y')
    optstring = optstring%"* "
  endif
  optstring = optstring%"| Draw Ground(toggle) "
  if (_drawground = 'y')
    optstring = optstring%"* "
  endif
  optstring = optstring%"| Plot Grid Values(toggle) "
  if (_gridvals = 'y')
    optstring = optstring%"* "
  endif
  optstring = optstring%"| Write data Values(toggle; simple only) "
  if (_writedata = 'y')
    optstring = optstring%"* "
  endif
  optstring = optstring%"| Set Grid Value Opts >33> "
  optstring = optstring%"| Set Vector Display Opts >34> "
  optstring = optstring%"| Set Azimuthal Ave Opts >35> "
  optstring = optstring%"| Set Streamline Density "
  
  _optmenu.1  = "setlevs"
  _optmenu.2  = "scheme"
  _optmenu.3  = "overlay"
  _optmenu.4  = "shade"
  _optmenu.5  = "labels"
  _optmenu.6  = "zlog"
  _optmenu.7  = "drawground"
  _optmenu.8  = "gridvals"
  _optmenu.9  = "writedata"
  _optmenu.10  = "gridopts"
  _optmenu.11 = "vectopts"
  _optmenu.12 = "azaveopts"
  _optmenu.13 = "strmlnden"
    
  schmstring = "Standard Scheme(toggle) "
  if (_stndrdschm = 'y')
    schmstring = schmstring%"* "
  endif
  schmstring = schmstring%"| Set Contour Interval | Set Color >32> "
  
  _schmmenu.1  = "standard"
  _schmmenu.2  = "setcint"
  _schmmenu.3  = "setcolor"
    
  colstring = "Rainbow "
  if (_defcclr = "rainbow")
    colstring = colstring%"* "
  endif
  colstring = colstring%"| black "
  if (_defcclr = 0)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| white "
  if (_defcclr = 1)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| red "
  if (_defcclr = 2)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| green "
  if (_defcclr = 3)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| dark blue "
  if (_defcclr = 4)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| light blue "
  if (_defcclr = 5)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| magenta "
  if (_defcclr = 6)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| yellow "
  if (_defcclr = 7)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| orange "
  if (_defcclr = 8)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| purple "
  if (_defcclr = 9)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| yellow/green "
  if (_defcclr = 10)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| medium blue "
  if (_defcclr = 11)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| dark yellow "
  if (_defcclr = 12)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| aqua "
  if (_defcclr = 13)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| dark purple "
  if (_defcclr = 14)
    colstring = colstring%"* "
  endif
  colstring = colstring%"| gray "
  if (_defcclr = 15)
    colstring = colstring%"* "
  endif
  
  _colmenu.1 = "rainbow"
  _colmenu.2 = 0
  _colmenu.3 = 1
  _colmenu.4 = 2
  _colmenu.5 = 3
  _colmenu.6 = 4
  _colmenu.7 = 5
  _colmenu.8 = 6
  _colmenu.9 = 7
  _colmenu.10 = 8
  _colmenu.11 = 9
  _colmenu.12 = 10
  _colmenu.13 = 11
  _colmenu.14 = 12
  _colmenu.15 = 13
  _colmenu.16 = 14
  _colmenu.17 = 15
  
  grdoptstring = "Grid Display Spacing | Grid Display Digits | Grid Display Size"

  _gopmenu.1 = "grddispspcing"
  _gopmenu.2 = "grddispdigits"
  _gopmenu.3 = "grddispsize"

  vecoptstring = "Vector Size | Vector Spacing"

  _vopmenu.1 = "vecsize"
  _vopmenu.2 = "vecspcing"

  azoptstring = "# of Radial Spokes to Ave | Radial Resolution (# of rings)"

  _azopmenu.1 = "nazimuth"
  _azopmenu.2 = "nrange"

  'clear dropmenu 3'
  'draw dropmenu 3 3.89 8.35 1.60 0.25 Options | 'optstring

  'clear dropmenu 31'
  'draw dropmenu 31 cascade 'schmstring

  'clear dropmenu 32'
  'draw dropmenu 32 cascade 'colstring

  'clear dropmenu 33'
  'draw dropmenu 33 cascade 'grdoptstring

  'clear dropmenu 34'
  'draw dropmenu 34 cascade 'vecoptstring

  'clear dropmenu 35'
  'draw dropmenu 35 cascade 'azoptstring

return

*------------------------*
*
*------------------------*
function makeunitmenu()
  
  unitstring = "Temperature >41> | Wind >42> "
  _unitmenu.1 = "temp"
  _unitmenu.2 = "wind"
  
  tempstring = "deg F "
  if (_tempunit = "degf")
    tempstring = tempstring%"* "
  endif
  
  tempstring = tempstring%"| deg C "
  if (_tempunit = "degc")
    tempstring = tempstring%"* "
  endif
  
  tempstring = tempstring%"| deg K "
  if (_tempunit = "degk")
    tempstring = tempstring%"* "
  endif
  
  _tempmenu.1 = "degf"
  _tempmenu.2 = "degc"
  _tempmenu.3 = "degk"
    
  windstring = "knots "
  if (_windunit = "knots")
    windstring = windstring%"* "
  endif
  
  windstring = windstring%"| m/s "
  if (_windunit = "m/s")
    windstring = windstring%"* "
  endif
  
  windstring = windstring%"| mph "
  if (_windunit = "mph")
    windstring = windstring%"* "
  endif

  _windmenu.1 = "knots"
  _windmenu.2 = "m/s"
  _windmenu.3 = "mph"
  
  'clear dropmenu 4'
  'draw dropmenu 4 5.46 8.35 1.00 0.25 Units | 'unitstring
  
  'clear dropmenu 41'
  'draw dropmenu 41 cascade 'tempstring
    
  'clear dropmenu 42'
  'draw dropmenu 42 cascade 'windstring
  
return

*------------------------*
*
*------------------------*
function loadctl()

  if (_nest = "6thdeg")
    _ctlfile = _stormid%"."%_itime%".6th.ctl"
    tmpt = (_fhour / 6) + 1
  else; if (_nest = "1deg")
    _ctlfile = _stormid%"."%_itime%".1deg.ctl"
    tmpt = (_fhour / 6) + 1
  else; if (_nest = "12thdeg")
    _ctlfile = _stormid%"."%_itime%".f"%_fhour%".ctl"
    tmpt = 1
  endif; endif; endif
  
  'open '_ctlfile
  'set dfile 1'
  'set t 'tmpt

return

*-----------------------*
*
*-----------------------*
function output()

* This function creates an output file of either b&w postscript,
* color postscript or color GIF format, depending on the user 
* input from the MAIN button menu.

  odir = '/net/'%_userid%'/hurplots'
  

  say ' '
  say ' OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO'
  say ' OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO'
  say ' '
  say ' Output file is being written.  Please wait....'
  say ' '

  ifile = odir%'/hur.grads.meta'
  'enable print 'ifile
  'print'
  'disable print'

  if (_hardcopy = "printbw")
    ofile = odir%'/'%_stormid%'.xsect.'%_itime%'.f'%_fhour%'.bw.ps'
    '!/net/tpm/grads/linux/gxps -i 'ifile' -o 'ofile
*    '!/net/tpm/grads/1.7.beta.12/gxps -i 'ifile' -o 'ofile
  endif
  if (_hardcopy = "printcl")
    ofile = odir%'/'%_stormid%'.xsect.'%_itime%'.f'%_fhour%'.ps'
    '!/net/tpm/grads/linux/gxps -c -i 'ifile' -o 'ofile
  endif
  if (_hardcopy = "printgif")
    ofile = odir%'/'%_stormid%'.xsect.'%_itime%'.f'%_fhour%'.gif'
*    '!/net/tpm/grads/1.7.beta.12/gxgif -r -x 757 -y 585 'ifile' -o 'ofile
    '!/net/tpm/grads/linux/gxgif -x 833 -y 644 'ifile' -o 'ofile
  endif
  if (_hardcopy = "printrgif")
    ofile = odir%'/'%_stormid%'.xsect.'%_itime%'.f'%_fhour%'.gif'
    '!/net/tpm/grads/linux/gxgif -r -x 833 -y 644 'ifile' -o 'ofile
  endif
  if (_hardcopy = "printpng")
    ofile = odir%'/'%_stormid%'.xsect.'%_itime%'.f'%_fhour%'.png'
    'printim 'ofile' x1000 y773 black'
  endif
  if (_hardcopy = "printrpng")
    ofile = odir%'/'%_stormid%'.xsect.'%_itime%'.f'%_fhour%'.png'
    'printim 'ofile' x1000 y773 white'
  endif
  say ' '
  say ' Output image has been written to the following file: '
  say ' '
  say ' 'ofile
  say ' '
  say ' OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO'
  say ' OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO'
  say ' '

return

*----------------------*
*                      *
*----------------------*
function convtime(vtime)

* Use this function to convert a date from the format of, for
* example, 12Z13FEB1998 to 98021312.  The reason I specifically
* choose that hhZddMONYYYY format is because that's what you
* get from the output of 'q dims'.

hh=substr(vtime,1,2)
yy=substr(vtime,9,4)
dd=substr(vtime,4,2)

cmon=substr(vtime,6,3)
if (cmon = 'JAN' | cmon = 'jan'); vplottim=yy%01%dd%hh; endif
if (cmon = 'FEB' | cmon = 'feb'); vplottim=yy%02%dd%hh; endif
if (cmon = 'MAR' | cmon = 'mar'); vplottim=yy%03%dd%hh; endif
if (cmon = 'APR' | cmon = 'apr'); vplottim=yy%04%dd%hh; endif
if (cmon = 'MAY' | cmon = 'may'); vplottim=yy%05%dd%hh; endif
if (cmon = 'JUN' | cmon = 'jun'); vplottim=yy%06%dd%hh; endif
if (cmon = 'JUL' | cmon = 'jul'); vplottim=yy%07%dd%hh; endif
if (cmon = 'AUG' | cmon = 'aug'); vplottim=yy%08%dd%hh; endif
if (cmon = 'SEP' | cmon = 'sep'); vplottim=yy%09%dd%hh; endif
if (cmon = 'OCT' | cmon = 'oct'); vplottim=yy%10%dd%hh; endif
if (cmon = 'NOV' | cmon = 'nov'); vplottim=yy%11%dd%hh; endif
if (cmon = 'DEC' | cmon = 'dec'); vplottim=yy%12%dd%hh; endif

return vplottim

*----------------------*
*                      *
*----------------------*
function testrhbug()

* This function tests if relative humididy is stored as a fraction.
* If it is 1 is returned, otherwise 0 is returned.

  'set gxout stat'
  'set lev 850'
  'd rh'
  
  rec = sublin(result, 8)
  max = subwrd(rec, 5)
    
  if (max <= 2)
    rhbug = 1
  else
    rhbug = 0
  endif
  
return rhbug

*----------------------*
*                      *
*----------------------*
function formatlon(lon)

* this function formats longitudes so that they will
* display as positeve number between 0 and 180 east or
* west

  if(lon < 0.0)
    while(lon < 0.0)
      lon = lon + 360.0
    endwhile
  endif
  if (lon >= 360.0)
    while(lon >= 360.0)
      lon = lon - 360.0
    endwhile
  endif

  if (lon >= 180.0)
    lon = 360.0 - lon
    if (lon >= 100.00)
      flon = math_format("%6.2f",lon)
    else 
      if (lon >= 10.0)
        flon = math_format("%5.2f",lon)
      else
        flon = math_format("%4.2f",lon)
      endif 
    endif
    label = flon%"W"
  else
    if (lon >= 100.00)
      flon = math_format("%6.2f",lon)
    else 
      if (lon >= 10.0)
        flon = math_format("%5.2f",lon)
      else
        flon = math_format("%4.2f",lon)
      endif 
    endif
    label = flon%"E"
  endif

return label

*----------------------*
*                      *
*----------------------*
function formatlat(lat)

* this function formats latitudes so that they will
* display as a positive number north or south.

  if(lat < 0.0)
    lat = -lat
    if (lat >= 10.0)
      flat = math_format("%5.2f",lat)
    else
      flat = math_format("%4.2f",lat)
    endif
    label = flat%"S"
  else
    if (lat >= 10.0)
      flat = math_format("%5.2f",lat)
    else
      flat = math_format("%4.2f",lat)
    endif
    label = flat%"N"
  endif
  
return label

*----------------------*
*                      *
*----------------------*
function db2ll(xlato,xlono,dist,bear)
 
*---------------------------------------------------------------------*
*                                                                     *
* Original script/formula from Albion D. Taylor.                      *
*                                                                     *
* Given an origin at latitude,longitude = xlato,xlono, this function  *
* will locate a target point at a distance dist in degrees, at        *
* bearing bear (degrees clockwise from north). Returns latitude       *
* xlatt and longitude xlont of target point.                          *
*                                                                     *
* All latitudes are in degrees, north positive and south negative.    *
* All longitudes are in degrees, east positive and west negative.     *
*                                                                     *
*---------------------------------------------------------------------*

  radpdeg = 0.017453293

  temp = dist*radpdeg
  cdist = math_cos(temp)
  sdist = math_sin(temp)

  temp = xlato*radpdeg
  clato = math_cos(temp)
  slato = math_sin(temp)

  temp = xlono*radpdeg
  clono = math_cos(temp)
  slono = math_sin(temp)

  temp = bear*radpdeg
  cbear = math_cos(temp)
  sbear = math_sin(temp

  z = cdist*slato + clato*sdist*cbear
  y = clato*clono*cdist + sdist*(-slono*sbear - slato*clono*cbear)
  x = -clato*slono*cdist - sdist*(clono*sbear - slato*slono*cbear)

  r = math_sqrt(x*x + y*y)

  xlatt = math_atan2(z,r)/radpdeg

  if (r <= 0) 
    xlont=0.
  else
    xlont = -math_atan2(x,y)/radpdg
  endif

return 'xlont = 'xlont' xlatt = 'xlatt

*----------------------*
*                      *
*----------------------*
function db2llfast(clato, slato, clono, slono, cdist, sdist, cbear, sbear)

* This is a version of db2ll that takes already computed sines and cosines
* as arguments.  This makes it more efficient when used in a loop.

  radpdeg = 0.017453293

  z = cdist*slato + clato*sdist*cbear
  y = clato*clono*cdist + sdist*(-slono*sbear - slato*clono*cbear)
  x = -clato*slono*cdist - sdist*(clono*sbear - slato*slono*cbear)

  r = math_sqrt(x*x + y*y)

  xlatt = math_atan2(z,r)/radpdeg

  if (r <= 0) 
    xlont=0.
  else
    xlont = -math_atan2(x,y)/radpdeg
  endif

return 'xlont = 'xlont' xlatt = 'xlatt
